@@ -89,6 +89,10 @@ int burn_setup_drive(struct burn_drive *d, char *fname)
         d->stream_recording_start= 0;
 	d->role_5_nwa = 0;
 	d->features = NULL;
+	d->drive_serial_number = NULL;
+	d->drive_serial_number_len = -1;
+	d->media_serial_number = NULL;
+	d->media_serial_number_len = -1;
 	return 1;
 }
 
@@ -114,6 +118,10 @@ void burn_drive_free_subs(struct burn_drive *d)
 		close (d->stdio_fd);
 	d->stdio_fd = -1;
 	burn_feature_descr_free(&(d->features), 0);
+	BURN_FREE_MEM(d->drive_serial_number);
+	BURN_FREE_MEM(d->media_serial_number);
+        d->drive_serial_number = d->media_serial_number = NULL;
+	d->drive_serial_number_len = d->media_serial_number_len = 0;
 	sg_dispose_drive(d, 0);
 }
 
@@ -411,6 +419,7 @@ int burn_drive_grab_stdio(struct burn_drive *d, int flag)
 		/* despite its name : last valid address, not size */
 		d->media_read_capacity =
 					read_size / 2048 - !(read_size % 2048);
+		d->mr_capacity_trusted = 1;
 		if ((stat_ret == -1 || is_rdwr) && d->devname[0]) { 
 		       	ret = burn_os_stdio_capacity(d->devname, 0, &size);
 			if (ret > 0)
@@ -704,6 +713,14 @@ int burn_drive_release_fl(struct burn_drive *d, int flag)
 	}
 
 	d->needs_sync_cache = 0; /* just to be sure */
+
+	if (d->drive_serial_number != NULL)
+		BURN_FREE_MEM(d->drive_serial_number);
+	if (d->media_serial_number != NULL)
+		BURN_FREE_MEM(d->media_serial_number);
+	d->drive_serial_number = d->media_serial_number = NULL;
+	d->drive_serial_number_len = d->media_serial_number_len = 0;
+
 	d->released = 1;
 
 	/* ts A61125 : outsourced model aspects */
@@ -1766,10 +1783,12 @@ int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 		strcpy(d->current_profile_text,"stdio file");
 		d->current_is_cd_profile = 0;
 		d->current_is_supported_profile = 1;
-		if (read_size >= 0)
+		if (read_size >= 0) {
 			/* despite its name : last valid address, not size */
 			d->media_read_capacity =
 				read_size / 2048 - !(read_size % 2048);
+			d->mr_capacity_trusted = 1;
+		}
 		burn_drive_set_media_capacity_remaining(d, size);
 	} else
 		d->current_profile = 0; /* Drives return this if empty */
@@ -1881,14 +1900,15 @@ int burn_drive_adr_debug_msg(char *fmt, char *arg)
 	int ret;
 	char *msg = NULL, *msgpt;
 
-	BURN_ALLOC_MEM(msg, char, 4096);
-	msgpt = msg;
-	if(arg != NULL)
-		sprintf(msg, fmt, arg);
-	else
-		msgpt = fmt;
 	if(libdax_messenger == NULL)
 		return 0;
+	if(arg != NULL) {
+		BURN_ALLOC_MEM(msg, char, 4096);
+		msgpt = msg;
+		sprintf(msg, fmt, arg);
+	} else {
+		msgpt = fmt;
+	}
 	ret = libdax_msgs_submit(libdax_messenger, -1, 0x00000002,
 				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_ZERO,
 				msgpt, 0, 0);
@@ -2865,7 +2885,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 		size = d->media_capacity_remaining;
 		burn_os_stdio_capacity(d->devname, 0, &size);
 		burn_drive_set_media_capacity_remaining(d, size);
-		o->start_range_high = d->media_capacity_remaining;
+		o->start_range_high = d->media_capacity_remaining - 2048;
 		o->start_alignment = 2048; /* imposting a drive, not a file */
 		o->might_do_sao = 4;
 		o->might_do_tao = 2;
@@ -2880,7 +2900,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 
 		/* >>> start_range_low = file size rounded to 2048 */;
 
-		o->start_range_high = d->media_capacity_remaining;
+		o->start_range_high = d->media_capacity_remaining - 2048;
 		o->start_alignment = 2048; /* imposting a drive, not a file */
 		if (s == BURN_DISC_APPENDABLE) {
 			if (wt == BURN_WRITE_SAO || wt == BURN_WRITE_RAW)
@@ -2961,7 +2981,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 					&num_formats);
 		if (ret == 1) {
 			if (status == BURN_FORMAT_IS_FORMATTED)
-				o->start_range_high = size;
+				o->start_range_high = size - 2048;
 			if (d->current_profile == 0x13) {
 				o->start_alignment = 32 * 1024;
 				for (i = 0; i < num_formats; i++) {
@@ -3305,7 +3325,8 @@ int burn_drive_set_media_capacity_remaining(struct burn_drive *d, off_t value)
 /* ts A81215 : API */
 int burn_get_read_capacity(struct burn_drive *d, int *capacity, int flag)
 {
-	*capacity = d->media_read_capacity + 1;
+	*capacity = d->media_read_capacity +
+			(d->media_read_capacity != 0x7fffffff);
 	return (d->media_read_capacity != 0x7fffffff);
 }
 
@@ -3432,7 +3453,7 @@ int burn_feature_descr_new(struct burn_feature_descr **new,
 	*new = NULL;
 	if (descr_len < 4)
 		return 0;
-	(*new) = o = calloc(1, sizeof(struct burn_speed_descriptor));
+	(*new) = o = calloc(1, sizeof(struct burn_feature_descr));
 	if (o == NULL)
 		return -1;
 	o->feature_code = (descr[0] << 8) | descr[1];
@@ -3490,3 +3511,69 @@ int burn_drive_has_feature(struct burn_drive *d, int feature_code,
 }
 
 
+/* ts B51016 API */
+int burn_drive_get_serial_no(struct burn_drive *d, char **sno, int *sno_len)
+{
+	int ret;
+
+	if (*sno != NULL)
+		BURN_FREE_MEM(*sno);
+	if (d->drive_serial_number_len > 0)
+		*sno_len = d->drive_serial_number_len;
+	else
+		*sno_len = 0;
+	BURN_ALLOC_MEM(*sno, char, *sno_len + 1);
+	if (d->drive_serial_number_len > 0)
+		memcpy(*sno, d->drive_serial_number, *sno_len);
+	(*sno)[*sno_len] = 0;
+	ret = 1;
+ex:
+	return ret;
+}
+
+
+/* ts B51016 API */
+int burn_drive_get_media_sno(struct burn_drive *d, char **sno, int *sno_len)
+{
+	int ret;
+
+#ifdef Libburn_enable_scsi_cmd_ABh
+	struct burn_feature_descr *feat;
+#endif
+
+	if (*sno != NULL)
+		BURN_FREE_MEM(*sno);
+	*sno = NULL;
+
+	if (d->media_serial_number_len == -1) {
+
+#ifdef Libburn_enable_scsi_cmd_ABh
+
+		if (burn_drive_has_feature(d, 0x109, &feat, 0))
+
+#ifndef Libburn_enable_scsi_cmd_ABh_pretend_currenT
+			if (feat->flags & 1) /* current */
+#endif
+
+				spc_read_media_serial_number(d);
+
+#else
+		;
+
+#endif /* ! Libburn_enable_scsi_cmd_ABh */
+
+	}
+
+	if (d->media_serial_number_len > 0)
+		*sno_len = d->media_serial_number_len;
+	else
+		*sno_len = 0;
+	BURN_ALLOC_MEM(*sno, char, *sno_len + 1);
+	if (*sno_len > 0)
+		memcpy(*sno, d->media_serial_number, *sno_len);
+	(*sno)[*sno_len] = 0;
+	ret = 1;
+ex:
+	return ret;
+}
+
