@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2014 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2016 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -253,6 +253,16 @@ struct burn_drive
 	*/
 	int current_feat2fh_byte4;
 
+	/* ts B51016 : Result from feature 108h : Drive Serial Number
+	*/
+	char *drive_serial_number;
+	int drive_serial_number_len;
+
+	/* ts B51016 : Result from command AB READ MEDIA SERIAL NUMBER
+	*/
+	char *media_serial_number;
+	int media_serial_number_len;
+
 	/* ts B10524 : whether the damage bit was set for the future track.
 	               bit0= damage bit , bit1= nwa valid bit
 	*/
@@ -309,6 +319,7 @@ struct burn_drive
 	   1= do not report errors
 	   2= do not report errors which the libburn function indicates in
 	      member .had_particular_error
+	   3= report errors with severity DEBUG
 	*/
 	int silent_on_scsi_error;
 
@@ -358,6 +369,11 @@ struct burn_drive
 	               0x7ffffff0 = 32 bit overflow, or unknown stdio size
 	 */
 	int media_read_capacity;
+	/* ts B60305 : Whether READ CAPACITY of CD is credible:
+	               -1= no READ CAPACITY yet , 0= untrusted READ CAPACITY
+	               1= READ CAPACITY confirmed or corrected by other commands
+	*/
+	int mr_capacity_trusted;
 
 	/* ts B10314 : Next Writeable Adress for drive_role == 5 */
         int role_5_nwa;
