@@ -2,7 +2,7 @@
 /*
   Class struct of libisoburn.
 
-  Copyright 2007 - 2015 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
+  Copyright 2007 - 2016 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
                     and Thomas Schmitt <scdbackup@gmx.net>
   Provided under GPL version 2 or later.
 */
@@ -112,7 +112,7 @@ struct isoburn {
  */
  enum burn_disc_status fabricated_disc_status;
 
- /* To be set if read errors occured during media evaluation.
+ /* To be set if read errors occurred during media evaluation.
  */
  int media_read_error;
 
@@ -702,6 +702,14 @@ struct isoburn_imgen_opts {
     */
     int appended_as_gpt;
 
+    /* If 1: With appended partitions: mark by APM
+    */
+    int appended_as_apm;
+
+    /* If 1: Apply isohybrid gestures to non-isohybrid situations
+    */
+    int part_like_isohybrid;
+
     /* Eventual name of the non-ISO aspect of the image. E.g. SUN ASCII label.
      */
     char ascii_disc_label[Libisoburn_disc_label_sizE];
