@@ -7,7 +7,7 @@
  See libisofs/aaip_0_2.h
      http://libburnia-project.org/wiki/AAIP
 
- Copyright (c) 2009 - 2014 Thomas Schmitt, libburnia project, GPLv2+
+ Copyright (c) 2009 - 2015 Thomas Schmitt, libburnia project, GPLv2+
 
 */
 
@@ -62,12 +62,12 @@
 #define Aaip_namespace_trusteD   0x05
 #define Aaip_namespace_securitY  0x06
 
-static char Aaip_namespace_textS[][16]= {"", "", "system.", "user.", "isofs.",
-                                         "trusted.", "security."};
-
 /* maximum expansion:  "security." */
 #define Aaip_max_name_expansioN  9
 
+static char Aaip_namespace_textS[][Aaip_max_name_expansioN + 1]=
+               {"", "", "system.", "user.", "isofs.", "trusted.", "security."};
+
 /* --------------------------------- Encoder ---------------------------- */
 
 
@@ -88,26 +88,30 @@ static int aaip_encode_pair(char *name, size_t attr_length, char *attr,
                         no longer needed 
    @param flag          Bitfield for control purposes
                         bit0= set CONTINUE bit of last AAIP field to 1
-   @return              >0 is the number of SUSP fields generated,
-                        0 means error 
+   @return              >= 0 is the number of SUSP fields generated,
+                        < 0 means error 
 */
-size_t aaip_encode(size_t num_attrs, char **names,
-                   size_t *value_lengths, char **values, 
-                   size_t *result_len, unsigned char **result, int flag)
+ssize_t aaip_encode(size_t num_attrs, char **names,
+                    size_t *value_lengths, char **values, 
+                    size_t *result_len, unsigned char **result, int flag)
 {
- size_t mem_size= 0, comp_size, ret;
+ size_t mem_size= 0, comp_size;
+ ssize_t ret;
  unsigned int number_of_fields, i, num_recs;
 
  /* Predict memory needs, number of SUSP fields and component records */
+ *result = NULL;
  *result_len= 0;
  for(i= 0; i < num_attrs; i++) {
    ret= aaip_encode_pair(names[i], value_lengths[i], values[i],
                          &num_recs, &comp_size, NULL, (size_t) 0, 1);
-   if(ret <= 0)
+   if(ret < 0)
      return(ret);
    mem_size+= comp_size;
  }
  number_of_fields= mem_size / 250 + !!(mem_size % 250);
+ if(number_of_fields == 0)
+   return(0);
  mem_size+= number_of_fields * 5;
 
 #ifdef Aaip_encode_debuG
@@ -118,14 +122,18 @@ size_t aaip_encode(size_t num_attrs, char **names,
 #endif
 
  if(*result == NULL)
-   return 0;
+   return ISO_OUT_OF_MEM;
 
  /* Encode pairs into result */
  for(i= 0; i < num_attrs; i++) {
    ret= aaip_encode_pair(names[i], value_lengths[i], values[i],
                          &num_recs, &comp_size, *result, *result_len, 0);
-   if(ret <= 0)
+   if(ret < 0) {
+     free(*result);
+     *result = NULL;
+     *result_len = 0;
      return(ret);
+   }
    (*result_len)+= comp_size;
  }
 
@@ -404,7 +412,8 @@ static ssize_t aaip_encode_acl_text(char *acl_text, mode_t st_mode,
          /* >>> Duplicate u:: entry. */;
          /* >>> ??? If it matches the previous one: ignore */
 
-         return((int) ISO_AAIP_ACL_MULT_OBJ);
+         ret = ISO_AAIP_ACL_MULT_OBJ;
+         goto ex;
        }
        has_u++;
      } else {
@@ -449,7 +458,8 @@ static ssize_t aaip_encode_acl_text(char *acl_text, mode_t st_mode,
          /* >>> Duplicate g:: entry. */;
          /* >>> ??? If it matches the previous one: ignore */
 
-         return((int) ISO_AAIP_ACL_MULT_OBJ);
+         ret = ISO_AAIP_ACL_MULT_OBJ;
+         goto ex;
        }
        has_g++;
      } else {
@@ -493,7 +503,8 @@ static ssize_t aaip_encode_acl_text(char *acl_text, mode_t st_mode,
        /* >>> Duplicate o:: entry. */;
        /* >>> ??? If it matches the previous one: ignore */
 
-       return((int) ISO_AAIP_ACL_MULT_OBJ);
+       ret = ISO_AAIP_ACL_MULT_OBJ;
+       goto ex;
      }
      has_o++;
    } else if(strncmp(rpt, "mask:", 5) == 0) {
@@ -1803,8 +1814,8 @@ int aaip_decode_attrs(struct aaip_state **handle,
    if(aaip->list_mem_used >= memory_limit)
      return(3);
    aaip->list_mem_used+= new_mem;
-   aaip->name_buf= calloc(sizeof(char *), Aaip_initial_name_leN);
-   aaip->value_buf= calloc(sizeof(char *), Aaip_initial_value_leN);
+   aaip->name_buf= calloc(1, Aaip_initial_name_leN);
+   aaip->value_buf= calloc(1, Aaip_initial_value_leN);
    if(aaip->name_buf == NULL || aaip->value_buf == NULL)
      return(-1);
    aaip->name_buf_size= Aaip_initial_name_leN;
@@ -2181,6 +2192,22 @@ ex:;
 
 #include "aaip-os-freebsd.c"
 
+#else
+#ifdef __FreeBSD_kernel__
+
+#ifdef NIX
+#ifdef Libisofs_with_aaip_xattR
+/* ts B51213: xattr system library calls are only stubs */
+#include "aaip-os-linux.c"
+#else
+/* ts B51213: extattr system library calls are not even present */
+#include "aaip-os-freebsd.c"
+#endif /* ! Libisofs_with_aaip_xattR */
+#else /* NIX */
+/* ts B51213: so we still end up at the dummy */
+#include "aaip-os-dummy.c"
+#endif /* ! NIX */
+
 #else
 #ifdef __NetBSD__
 
@@ -2204,5 +2231,6 @@ ex:;
 
 #endif /* ! __linux */
 #endif /* ! __NetBSD__ */
+#endif /* ! __FreeBSD_kernel__ */
 #endif /* ! __FreeBSD__ */
 
