@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 - 2013 Thomas Schmitt
+ * Copyright (c) 2009 - 2015 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -468,17 +468,33 @@ struct iso_write_opts {
        to HFS+/FAT and IsoFileSrc areas and marked by an MBR partition entry.
     */
     char *prep_partition;
+    int prep_part_flag;
 
     /* Eventual disk file path of an EFI system partition image which shall
        be prepended to HFS+/FAT and IsoFileSrc areas and marked by a GPT entry.
     */
     char *efi_boot_partition;
+    int efi_boot_part_flag;
 
     /* Eventual disk file paths of prepared images which shall be appended
        after the ISO image and described by partiton table entries in a MBR
     */
     char *appended_partitions[ISO_MAX_PARTITIONS]; 
     uint8_t appended_part_types[ISO_MAX_PARTITIONS];
+    int appended_part_flags[ISO_MAX_PARTITIONS];
+
+    /* If 1: With appended partitions: create protective MBR and mark by GPT
+     */
+    int appended_as_gpt;
+
+    /* If 1: With appended partitions: mark by APM partition
+     */
+    int appended_as_apm;
+
+    /* If 1: Obey struct el_torito_boot_image.isolinux_options bit2-7 and bit8.
+             I.e. mention boot image as partition in GPT and/or APM.
+     */
+    int part_like_isohybrid;
 
     /* Eventual name of the non-ISO aspect of the image. E.g. SUN ASCII label.
      */
@@ -547,10 +563,19 @@ struct ecma119_image
 
     time_t now; /**< Time at which writing began. */
 
-    /** Total size of the output. This only includes the current volume. */
+    /* Total size of the output. Counted in bytes.
+     * Includes ISO filesystem and appended data.
+     */
     off_t total_size;
+
+    /** Size actually governed by the ISO filesystem part of the output */
     uint32_t vol_space_size;
 
+    /* 1= write the total size into the PVD of the ISO,
+     * 0= write vol_space_size
+     */
+    int pvd_size_is_total_size;
+
     /* Bytes already written to image output */
     off_t bytes_written;
     /* just for progress notification */
@@ -633,6 +658,11 @@ struct ecma119_image
     int num_bootsrc;
     IsoFileSrc **bootsrc; /* location of the boot images in the new image */
 
+    int *boot_appended_idx; /* Appended partition which serve as boot images */
+
+    uint32_t *boot_intvl_start;   /* In blocks of 2048 bytes */
+    uint32_t *boot_intvl_size;    /* In blocks of  512 bytes */
+
     /*
      * System Area related information
      */
@@ -695,6 +725,8 @@ struct ecma119_image
     /* tree of files sources */
     IsoRBTree *files;
 
+    struct iso_filesrc_list_item *ecma119_hidden_list;
+
     unsigned int checksum_idx_counter;
     void *checksum_ctx;
     off_t checksum_counter;
@@ -756,6 +788,7 @@ struct ecma119_image
     uint32_t appended_part_prepad[ISO_MAX_PARTITIONS];
     uint32_t appended_part_start[ISO_MAX_PARTITIONS];
     uint32_t appended_part_size[ISO_MAX_PARTITIONS];
+    int have_appended_partitions;
 
     /* See IsoImage and libisofs.h */
     IsoNode *hfsplus_blessed[ISO_HFSPLUS_BLESS_MAX];
@@ -790,6 +823,12 @@ struct ecma119_image
     struct iso_mbr_partition_request *mbr_req[ISO_MBR_ENTRIES_MAX];
     int mbr_req_count;
 
+    /* Number of bytes which have to be added after the cylinder aligned end
+       of the overall ISO partition because clinder size is not a multiple
+       of 2048
+    */
+    int post_iso_part_pad;
+
     uint32_t prep_part_size;
 
     /* GPT description. To be composed during IsoImageWriter
@@ -802,6 +841,9 @@ struct ecma119_image
     /* bit0= GPT partitions may overlap */
     int gpt_req_flags;
 
+    /* Whether the eventual backup GPT is not part of the ISO filesystem */
+    int gpt_backup_outside;
+
     uint32_t efi_boot_part_size;
     IsoFileSrc *efi_boot_part_filesrc; /* Just a pointer. Do not free. */
 
@@ -822,6 +864,7 @@ struct ecma119_image
        write_data() methods of the writers.
     */
     uint8_t sys_area_as_written[16 * BLOCK_SIZE];
+    int sys_area_already_written;
 
     /* Size of the filesrc_writer area (data file content).
        This is available before any IsoImageWriter.compute_data_blocks()
