@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2011-2012 Thomas Schmitt
+ * Copyright (c) 2011-2014 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -80,7 +80,8 @@ void iso1999_node_free(Iso1999Node *node)
         for (i = 0; i < node->info.dir->nchildren; i++) {
             iso1999_node_free(node->info.dir->children[i]);
         }
-        free(node->info.dir->children);
+        if (node->info.dir->children != NULL)
+            free(node->info.dir->children);
         free(node->info.dir);
     }
     iso_node_unref(node->node);
@@ -111,11 +112,14 @@ int create_node(Ecma119Image *t, IsoNode *iso, Iso1999Node **node)
             free(n);
             return ISO_OUT_OF_MEM;
         }
-        n->info.dir->children = calloc(sizeof(void*), dir->nchildren);
-        if (n->info.dir->children == NULL) {
-            free(n->info.dir);
-            free(n);
-            return ISO_OUT_OF_MEM;
+        n->info.dir->children = NULL;
+        if (dir->nchildren > 0) {
+            n->info.dir->children = calloc(sizeof(void*), dir->nchildren);
+            if (n->info.dir->children == NULL) {
+                free(n->info.dir);
+                free(n);
+                return ISO_OUT_OF_MEM;
+            }
         }
         n->type = ISO1999_DIR;
     } else if (iso->type == LIBISO_FILE) {
@@ -125,7 +129,7 @@ int create_node(Ecma119Image *t, IsoNode *iso, Iso1999Node **node)
         IsoFile *file = (IsoFile*) iso;
 
         size = iso_stream_get_size(file->stream);
-        if (size > (off_t)MAX_ISO_FILE_SECTION_SIZE && t->iso_level != 3) {
+        if (size > (off_t)MAX_ISO_FILE_SECTION_SIZE && t->opts->iso_level != 3) {
             char *ipath = iso_tree_get_node_path(iso);
             ret = iso_msg_submit(t->image->id, ISO_FILE_TOO_BIG, 0,
                          "File \"%s\" can't be added to image because is "
@@ -194,7 +198,7 @@ int create_tree(Ecma119Image *t, IsoNode *iso, Iso1999Node **tree, int pathlen)
     }
 
     max_path = pathlen + 1 + (iso_name ? strlen(iso_name): 0);
-    if (!t->allow_longer_paths && max_path > 255) {
+    if (!t->opts->allow_longer_paths && max_path > 255) {
         char *ipath = iso_tree_get_node_path(iso);
         ret = iso_msg_submit(t->image->id, ISO_FILE_IMGPATH_WRONG, 0,
                      "File \"%s\" can't be added to ISO 9660:1999 tree, "
@@ -293,6 +297,8 @@ void sort_tree(Iso1999Node *root)
 {
     size_t i;
 
+    if (root->info.dir->children == NULL)
+        return;
     qsort(root->info.dir->children, root->info.dir->nchildren,
           sizeof(void*), cmp_node);
     for (i = 0; i < root->info.dir->nchildren; i++) {
@@ -308,14 +314,18 @@ int mangle_single_dir(Ecma119Image *img, Iso1999Node *dir)
     int ret;
     int i, nchildren;
     Iso1999Node **children;
-    IsoHTable *table;
+    IsoHTable *table = NULL;
     int need_sort = 0;
     char *full_name = NULL, *tmp = NULL;
 
+    nchildren = dir->info.dir->nchildren;
+    if (nchildren <= 0) {
+        ret = ISO_SUCCESS;
+        goto ex;
+    }
+    children = dir->info.dir->children;
     LIBISO_ALLOC_MEM(full_name, char, 208);
     LIBISO_ALLOC_MEM(tmp, char, 208);
-    nchildren = dir->info.dir->nchildren;
-    children = dir->info.dir->children;
 
     /* a hash table will temporary hold the names, for fast searching */
     ret = iso_htable_create((nchildren * 100) / 80, iso_str_hash,
@@ -722,13 +732,13 @@ void write_one_dir_record(Ecma119Image *t, Iso1999Node *node, int file_id,
     iso_bb(rec->block, block, 4);
     iso_bb(rec->length, len, 4);
 
-    /* was: iso_datetime_7(rec->recording_time, t->now, t->always_gmt);
+    /* was: iso_datetime_7(rec->recording_time, t->now, t->opts->always_gmt);
     */
     iso= node->node;
     iso_datetime_7(rec->recording_time, 
-                   (t->dir_rec_mtime & 4) ? ( t->replace_timestamps ?
-                                              t->timestamp : iso->mtime )
-                                          : t->now, t->always_gmt);
+                   (t->opts->dir_rec_mtime & 4) ? ( t->replace_timestamps ?
+                                                    t->timestamp : iso->mtime )
+                                                : t->now, t->opts->always_gmt);
 
     rec->flags[0] = ((node->type == ISO1999_DIR) ? 2 : 0) | (multi_extend ? 0x80 : 0);
     iso_bb(rec->vol_seq_number, (uint32_t) 1, 2);
