@@ -3,7 +3,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -207,6 +207,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->displacement_sign= 0;
  m->read_fs= 0;
  m->drives_exclusive= 1;
+ m->linux_scsi_dev_family= 0;
  m->early_stdio_test= 0;
  m->cache_num_tiles= 0;
  m->cache_tile_blocks= 0;
@@ -274,6 +275,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
 
  m->boot_efi_default= 0;
  m->system_area_disk_path[0]= 0;
+ m->system_area_clear_loaded= 0;
  m->system_area_options= 0;
  m->patch_system_area= 0;
  m->partition_offset= 0;
@@ -286,6 +288,8 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
    m->appended_part_types[i]= 0;
  }
  m->appended_as_gpt= 0;
+ m->appended_as_apm= 0;
+ m->part_like_isohybrid= 0;
  m->ascii_disc_label[0]= 0;
  m->grub2_sparc_core[0]= 0;
  memset(m->hfsp_serial_number, 0, 8);
