@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -85,7 +85,7 @@ handle_error:;
 handle_abort:;
      if(strcmp(lpt, link_path)!=0) {
        sprintf(xorriso->info_text,
-               "Problem occured with intermediate path : ");
+               "Problem occurred with intermediate path : ");
        Text_shellsafe(lpt, xorriso->info_text, 1);
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
      }
@@ -360,18 +360,15 @@ int Xorriso_show_dux_subs(struct XorrisO *xorriso,
  struct LinkiteM *own_link_stack;
  char *path= NULL, *show_path= NULL, *name= NULL, *sfe= NULL;
  
- sfe= malloc(5*SfileadrL);
- path= malloc(SfileadrL);
- show_path= malloc(SfileadrL);
- name= malloc(SfileadrL);
- if(path==NULL || show_path==NULL || name==NULL || sfe==NULL) {
-   Xorriso_no_malloc_memory(xorriso, &sfe, 0);
-   {ret= -1; goto ex;}
- }
  own_link_stack= link_stack;
  namept= name;
  *size= 0;
 
+ Xorriso_alloc_meM(sfe, char, 5 * SfileadrL);
+ Xorriso_alloc_meM(path, char, SfileadrL);
+ Xorriso_alloc_meM(show_path, char, SfileadrL);
+ Xorriso_alloc_meM(name, char, SfileadrL);
+
  if(lstat(abs_path, &stbuf)==-1)
    {ret= 2; goto ex;}
  dir_dev= stbuf.st_dev;
@@ -577,14 +574,10 @@ revoke_sorting:;
 
  ret= 1;
 ex:;
- if(sfe!=NULL)
-   free(sfe);
- if(path!=NULL)
-   free(path);
- if(show_path!=NULL)
-   free(show_path);
- if(name!=NULL)
-   free(name);
+ Xorriso_free_meM(sfe);
+ Xorriso_free_meM(path);
+ Xorriso_free_meM(show_path);
+ Xorriso_free_meM(name);
  Linkitem_reset_stack(&own_link_stack, link_stack, 0);
  Dirseq_destroy(&dirseq, 0);
  if(filev!=NULL) {
@@ -1161,8 +1154,6 @@ int Xorriso_findx_action(struct XorrisO *xorriso, struct FindjoB *job,
                                 &mode_and, &mode_or, &type, &date, &subjob, 0);
  if(action<0)
    action= 0;
- if(action<0)
-   action= 0;
  if(action==15 || action==16 || action==18 || action==19 || action==20) {
    /* in_iso , not_in_iso, add_missing , empty_iso_dir , is_full_in_iso */
    Findjob_get_start_path(job, &disk_prefix, 0);
