@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -54,6 +54,7 @@ int Xorriso_cdrskin_help(struct XorrisO *xorriso, int flag)
 "\t-version\tprint version information and exit emulation",
 "\t--devices\tprint list of available MMC drives and exit emulation",
 "\tdev=target\tpseudo-SCSI target to use as CD-Recorder",
+"\tdrive_scsi_dev_family=sr|scd|sg\t\tselect Linux device type",
 "\t-v\t\tincrement verbose level by one",
 "\t-V\t\tincrement SCSI command transport verbose level by one",
 "\t-checkdrive\tcheck if a driver for the drive is present",
@@ -78,6 +79,9 @@ int Xorriso_cdrskin_help(struct XorrisO *xorriso, int flag)
 "\t-isosize\tUse iso9660 file system size for next data track",
 "\t-pad\t\tpadsize=30k",
 "\t-nopad\t\tDo not pad",
+"\tminbuf=\t\tKeep drive buffer hungry",
+"\tmodesty_on_drive=\tLike minbuf=, but with more parameters",
+"\t-immed\t\tEquivalent to minbuf=75",
 "\t--grow_overwriteable_iso\temulate multi-session on DVD+RW, BD-RE",
 "\twrite_start_address=#\t\twrite to byte address on DVD+RW, BD-RE",
 "\tstream_recording=on|number\ttry to get full speed on DVD-RAM, BD",
@@ -120,10 +124,10 @@ int Xorriso_cdrskin(struct XorrisO *xorriso, char *whom, int argc, char **argv,
    "timeout=", "debug=", "kdebug=", "kd=", "driver=", "ts=",
    "pregap=", "defpregap=", "mcn=", "isrc=", "index=", "textfile=",
    "pktsize=", "cuefile=",
-   "gracetime=", "minbuf=",
+   "gracetime=",
 
    "assert_write_lba=", "fifo_start_at=", "dev_translation=",
-   "drive_scsi_dev_family=", "fallback_program=", "modesty_on_drive=",
+   "fallback_program=",
    "tao_to_sao_tsize=", 
    
    "direct_write_amount=", "msifile=",
@@ -332,6 +336,22 @@ no_volunteer:;
              strncmp(argpt, "stream_recording=", 17)==0) {
      cpt= strchr(argpt, '=')+1;
      Xorriso_option_stream_recording(xorriso, cpt, 0);
+
+   } else if(strcmp(argpt, "-immed") == 0) {
+     Xorriso_option_modesty_on_drive(xorriso, "75", 0);
+
+   } else if(strncmp(argpt, "-minbuf=", 8) == 0 ||
+             strncmp(argpt, "minbuf=", 7) == 0 ||
+             strncmp(argpt, "-modesty_on_drive=", 18) == 0 ||
+             strncmp(argpt, "modesty_on_drive=", 17) == 0) {
+     cpt= strchr(argpt, '=') + 1;
+     Xorriso_option_modesty_on_drive(xorriso, cpt, 0);
+
+   } else if(strncmp(argpt, "-drive_scsi_dev_family=", 23) == 0 ||
+             strncmp(argpt, "drive_scsi_dev_family=", 22) == 0) {
+     cpt= strchr(argpt, '=') + 1;
+     Xorriso_option_scsi_dev_family(xorriso, cpt, 0);
+
    } else if(strncmp(argpt, "-stdio_sync=", 12)==0 ||
              strncmp(argpt, "stdio_sync=", 11)==0) {
      cpt= strchr(argpt, '=') + 1;
@@ -598,7 +618,8 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "--no_rc", "--norock", "-hfsplus", "-fat", "-chrp-boot-part",
     "-isohybrid-gpt-basdat", "-isohybrid-gpt-hfsplus",
     "-isohybrid-apm-hfsplus", "--grub2-boot-info", "-joliet-utf16",
-    "-appended_part_as_gpt",
+    "-appended_part_as_gpt", "-appended_part_as_apm", "--mbr-force-bootable",
+    "-part_like_isohybrid",
     ""
  };
  static char arg1_options[][41]= {
@@ -637,7 +658,7 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "-rr_reloc_dir", "-hfsplus-serial-no", "-prep-boot-part", "-efi-boot-part",
     "-hfsplus-block-size", "-apm-block-size", "--grub2-mbr",
     "--grub2-sparc-core", "--sort-weight-list", "--sort-weight-patterns",
-    "-hppa-hdrversion",
+    "-hppa-hdrversion","-file_name_limit",
     ""
  };
  static char arg2_options[][41]= {
@@ -797,6 +818,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -R, -rock                   Generate Rock Ridge directory information",
 "  -r, -rational-rock          Generate rationalized Rock Ridge directory information",
 "  --norock                    Disable Rock Ridge. (Strongly discouraged !)",
+"  -file_name_limit LEN        Set truncation limit for Rock Ridge names",
 "  --hardlinks                 Record eventual hard link relations of files",
 "  --acl                       Record eventual ACLs of files",
 "  --xattr                     Record eventual user space xattr of files",
@@ -845,9 +867,11 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -isohybrid-gpt-basdat       Mark El Torito boot image as Basic Data in GPT",
 "  -isohybrid-gpt-hfsplus      Mark El Torito boot image as HFS+ in GPT",
 "  -isohybrid-apm-hfsplus      Mark El Torito boot image as HFS+ in APM",
+"  -part_like_isohybrid        Mark in MBR, GPT, APM without -isohybrid-mbr",
 "  -G FILE, -generic-boot FILE Set generic boot image name",
 "  --embedded-boot FILE        Alias of -G",
 "  --protective-msdos-label    Patch System Area by partition table",
+"  --mbr-force-bootable        Enforce existence of bootable flag in MBR",
 "  -partition_offset LBA       Make image mountable by first partition, too",
 "  -partition_sec_hd NUMBER    Define number of sectors per head",
 "  -partition_hd_cyl NUMBER    Define number of heads per cylinder",
@@ -872,6 +896,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -append_partition NUMBER TYPE FILE",
 "                              Append FILE after image. TYPE is hex: 0x..",
 "  -appended_part_as_gpt       mark appended partitions in GPT instead of MBR.",
+"  -appended_part_as_apm       mark appended partitions in APM.",
 "  --modification-date=YYYYMMDDhhmmsscc",
 "                              Override date of creation and modification",
 "  -isohybrid-mbr FILE         Set SYSLINUX mbr/isohdp[fp]x*.bin for isohybrid",
@@ -966,8 +991,6 @@ int Xorriso_genisofs_hide_list(struct XorrisO *xorriso, char *whom,
    return(0);
  }
  ret= Xorriso_afile_fopen(xorriso, adr, "rb", &fp, 0);
- if(ret <= 0)
-   return(0);
  if(ret <= 0)
    return(0);
  while(1) {
@@ -1119,6 +1142,74 @@ ex:;
 }
 
 
+/* Implementing mkisofs tendency to map single-path pathspecs to / */
+int Xorriso_graftable_pathspec(struct XorrisO *xorriso, char *in_pathspec,
+                               char *pathspec, int flag)
+{
+ int ret, l;
+ char *esc_wdx= NULL, *eff_path= NULL, *ept;
+
+ if((!xorriso->allow_graft_points) || 
+   Fileliste__target_source_limit(in_pathspec, '=', &ept, 0) <= 0) {
+
+   Xorriso_alloc_meM(esc_wdx, char, SfileadrL);
+   Xorriso_alloc_meM(eff_path, char, SfileadrL);
+
+   strcpy(esc_wdx, xorriso->wdx);
+   if(!xorriso->allow_graft_points) {
+     ret= Fileliste__escape_source_path(esc_wdx, SfileadrL, 0);
+     if(ret <= 0) {
+       Xorriso_msgs_submit(xorriso, 0,
+                 "Escaped -cdx directory gets much too long", 0, "FAILURE", 0);
+       ret= 0; goto ex;
+     }
+   }
+   ret= Xorriso_normalize_img_path(xorriso, esc_wdx, in_pathspec,
+                                   eff_path, 2|4);
+   if(ret <= 0)
+     {ret= 0; goto ex;}
+   ret= Sfile_type(eff_path,
+            1 | ((xorriso->do_follow_param || xorriso->do_follow_links) << 2));
+   if(ret == 2) {
+     strcpy(pathspec, "/=");
+   } else {
+     pathspec[0]= '/';
+     pathspec[1]= 0;
+     ret= Sfile_leafname(eff_path, pathspec + 1, 0);
+     if(ret>0) {
+       if(!xorriso->allow_graft_points) {
+         ret= Fileliste__escape_source_path(pathspec, SfileadrL, 0);
+         if(ret <= 0) {
+           Xorriso_msgs_submit(xorriso, 0,
+                      "Escaped leaf name gets much too long", 0, "FAILURE", 0);
+           ret= 0; goto ex;
+         }
+       }
+       strcat(pathspec, "=");
+     } else
+       pathspec[0]= 0;
+   }
+   l= strlen(pathspec);
+   strcat(pathspec, eff_path);
+   if(!xorriso->allow_graft_points) {
+     ret= Fileliste__escape_source_path(pathspec + l, 2 * SfileadrL - l, 0);
+     if(ret <= 0) {
+       Xorriso_msgs_submit(xorriso, 0,
+                           "Escaped path gets much too long", 0, "FAILURE", 0);
+       ret= 0; goto ex;
+     }
+   }
+ } else {
+   Sfile_str(pathspec, in_pathspec, 0);
+ }
+ ret= 1;
+ex:;
+ Xorriso_free_meM(esc_wdx);
+ Xorriso_free_meM(eff_path);
+ return(ret);
+}
+
+
 int Xorriso_genisofs_path_pecul(struct XorrisO *xorriso, int *was_path,
                                 int with_emul_toc, int *allow_dir_id_ext,
                                 int *iso_level, int flag)
@@ -1446,8 +1537,14 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
      xorriso->no_emul_toc|= 1;
    } else if(strcmp(argpt, "-log-file") == 0) {
      /* already handled before this loop */;
+     i++;
    } else if(strcmp(argpt, "-gui") == 0) {
      xorriso->pacifier_interval= 0.25;
+   } else if(strcmp(argpt, "-file_name_limit") == 0) {
+     if(i+1>=argc)
+       goto not_enough_args;
+     i++;
+     Xorriso_option_file_name_limit(xorriso, argv[i], 0);
    } else {
      if(argv[i][0] == '-') {
        ret= Xorriso_genisofs_fused_options(xorriso, whom, argv[i] + 1,
@@ -1591,6 +1688,7 @@ illegal_c:;
      /* was already handled in first argument scan */;
    } else if(strcmp(argpt, "--norock")==0) {
      xorriso->do_rockridge= 0;
+     lower_r= 0;
    } else if(strcmp(argpt, "-R")==0 || strcmp(argpt, "-rock")==0) {
      ret= Xorriso_genisofs_fused_options(xorriso, whom, "R",
                                   &option_d, &iso_level, &lower_r, ra_text, 0);
@@ -1654,7 +1752,7 @@ illegal_c:;
        goto problem_handler_2;
 
    } else if(strcmp(argpt, "-graft-points")==0) {
-     xorriso->allow_graft_points= 1;
+     xorriso->allow_graft_points= 3;
    } else if(strcmp(argpt, "-path-list")==0 ||
              strcmp(argpt, "--quoted_path_list")==0) {
      if(i+1>=argc) {
@@ -1667,7 +1765,7 @@ not_enough_args:;
      i++;
      xorriso->pacifier_style= 1;
      ret= Xorriso_option_path_list(xorriso, argv[i],
-                                   (strcmp(argpt, "--quoted_path_list")==0));
+                               (strcmp(argpt, "--quoted_path_list") == 0) | 2);
      if(ret<=0)
        goto problem_handler_2;
      ret = Xorriso_genisofs_path_pecul(xorriso, &was_path, with_emul_toc,
@@ -1772,10 +1870,12 @@ not_enough_args:;
              strncmp(argpt, "isolinux_mbr=", 13)==0 ||
              strcmp(argpt, "-eltorito-alt-boot")==0 ||
              strcmp(argpt, "--protective-msdos-label")==0 ||
+             strcmp(argpt, "--mbr-force-bootable")==0 ||
              strcmp(argpt, "--boot-catalog-hide")==0 ||
              strcmp(argpt, "-isohybrid-gpt-basdat")==0 ||
              strcmp(argpt, "-isohybrid-gpt-hfsplus")==0 ||
-             strcmp(argpt, "-isohybrid-apm-hfsplus")==0) {
+             strcmp(argpt, "-isohybrid-apm-hfsplus")==0 ||
+             strcmp(argpt, "-part_like_isohybrid")==0) {
      delay_opt_list[delay_opt_count++]= i;
      if(argv[i] != argpt)
        delay_opt_list[delay_opt_count - 1]|= 1<<31;
@@ -1970,6 +2070,9 @@ not_enough_args:;
    } else if(strcmp(argpt, "-appended_part_as_gpt") == 0) {
      xorriso->appended_as_gpt= 1;
 
+   } else if(strcmp(argpt, "-appended_part_as_apm") == 0) {
+     xorriso->appended_as_apm= 1;
+
    } else if(strcmp(argpt, "-B") == 0 ||
              strcmp(argpt, "-sparc-boot") == 0) {
      i++;
@@ -2083,9 +2186,8 @@ rr_reloc_dir:;
        goto problem_handler_2;
      Xorriso_relax_compliance(xorriso, "deep_paths_off:long_paths_off", 0);
 
-   } else if(strcmp(argpt, "-log-file") == 0) {
-     if(i + 1 >= argc)
-       goto not_enough_args;
+   } else if(strcmp(argpt, "-log-file") == 0 ||
+             strcmp(argpt, "-file_name_limit") == 0) {
      i+= 1;
      /* was already handled before this loop */;
 
@@ -2117,36 +2219,9 @@ rr_reloc_dir:;
        }
      }
    } else {
-     /* implementing mkisofs tendency to map single-path pathspecs to / */
-     if((!xorriso->allow_graft_points) || 
-        Fileliste__target_source_limit(argv[i], '=', &ept, 0)<=0) {
-       ret= Xorriso_normalize_img_path(xorriso, xorriso->wdx, argv[i],
-                                       eff_path, 2|4);
-       if(ret<=0)
-         goto problem_handler_2;
-       ret= Sfile_type(eff_path,
-                  1|((xorriso->do_follow_param||xorriso->do_follow_links)<<2));
-       if(ret==2) {
-         strcpy(pathspec, "/=");
-       } else {
-         pathspec[0]= '/';
-         pathspec[1]= 0;
-         ret= Sfile_leafname(eff_path, pathspec+1, 0);
-         if(ret>0) {
-           ret= Fileliste__escape_source_path(pathspec, SfileadrL, 0);
-           if(ret <= 0) {
-             Xorriso_msgs_submit(xorriso, 0,
-                                 "Escaped leaf name gets much too long",
-                                 0, "FAILURE", 0);
-             goto problem_handler_2;
-           }
-           strcat(pathspec, "=");
-         } else
-           pathspec[0]= 0;
-       }
-       strcat(pathspec, eff_path);
-     } else
-       Sfile_str(pathspec, argv[i], 0);
+     ret= Xorriso_graftable_pathspec(xorriso, argv[i], pathspec, 0);
+     if(ret <= 0)
+       goto problem_handler_2;
      add_pt= pathspec;
 
      if(old_root[0]) {
@@ -2187,10 +2262,10 @@ rr_reloc_dir:;
        ret= Xorriso_option_update(xorriso, disk_pt, iso_rr_pt, 1 | 8 | 32);
      } else {
        mem_graft_points= xorriso->allow_graft_points;
-       xorriso->allow_graft_points= 1;
+       xorriso->allow_graft_points= 3;
        zero= 0;
        ret= Xorriso_option_add(xorriso, 1, &add_pt, &zero,
-                               (was_path << 1) | (root_seen << 2) | 8);
+                               (was_path << 1) | (root_seen << 2));
        xorriso->allow_graft_points= mem_graft_points;
      }
      if(ret<=0)
@@ -2325,7 +2400,7 @@ problem_handler_2:;
          xorriso->boot_image_emul= 2;
      }
      boot_path[0]= 0;
-     if(argv[i][0] != '/')
+     if(argv[i][0] != '/' && strncmp(argv[i], "--interval:", 11) != 0)
        strcat(boot_path, "/");
      ret= Sfile_str(boot_path + strlen(boot_path), argv[i], 0);
      if(ret <= 0)
@@ -2382,6 +2457,10 @@ problem_handler_2:;
                                      (2 << 2);
    } else if(strcmp(argpt, "-isohybrid-apm-hfsplus") == 0) {
      xorriso->patch_isolinux_image = xorriso->patch_isolinux_image | (1 << 8);
+
+   } else if(strcmp(argpt, "-part_like_isohybrid") == 0) {
+     xorriso->part_like_isohybrid= 1;
+
    } else if(strcmp(argpt, "-eltorito-alt-boot")==0) {
      ret= Xorriso_genisofs_add_boot(xorriso, 0);
      if(ret <= 0)
@@ -2409,6 +2488,10 @@ problem_handler_2:;
                                   (xorriso->system_area_options & ~2) | 0x4000;
    } else if(strcmp(argpt, "--protective-msdos-label")==0) {
      xorriso->system_area_options= (xorriso->system_area_options & ~2) | 1;
+
+   } else if(strcmp(argpt, "--mbr-force-bootable") == 0) {
+     xorriso->system_area_options= xorriso->system_area_options | (1 << 15);
+
    } else if(strcmp(argpt, "--boot-catalog-hide")==0) {
      xorriso->boot_image_cat_hidden|= 3;
    } else if(strcmp(argpt, "-partition_offset") == 0 ||
