@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -52,6 +52,7 @@ struct ExprtesT {
   21= -bad_outname int namespace
   22= -use_pattern char *arg1 ("on" [or "ls"], "off")
   23= -or_use_pattern char *arg1 ("on" [or "ls"], "off")
+  24= -name_limit_blocker int *arg1
  */
  int test_type;
 
@@ -174,6 +175,13 @@ struct FindjoB {
    51= report_sections
    52= show_stream_id
    53= internal: show_hfs_cmd , controlled by xorriso->show_hfs_cmd*
+   54= internal: truncate_names
+   55= internal: unique_trunc_test lenght
+                 test for not uniquely truncatable names,
+                 result delivered in XorrisO.find_unique_trunc_result
+   56= like 54 but tolerating existing truncated names
+   57= like 55 but tolerating existing truncated names
+   58= internal: last_data_file_block
  */
  int action;
  int prune;
@@ -185,12 +193,14 @@ struct FindjoB {
  uid_t user;
  gid_t group;
  mode_t mode_and, mode_or;
- int type; /* see Xorriso_set_time flag, also used as weight */
+ int type; /* see Xorriso_set_time flag,
+              also used as weight and truncate_length */
  time_t date;
  char *found_path;
  off_t estim_upper_size;
  off_t estim_lower_size;
  struct FindjoB *subjob;
+ uint32_t last_data_file_block;
 
  /* Errors */
  char errmsg[4096];
@@ -357,6 +367,9 @@ int Findjob_set_found_path(struct FindjoB *o, char *path, int flag);
 
 int Findjob_get_found_path(struct FindjoB *o, char **path, int flag);
 
+int Findjob_get_last_data_file_block(struct FindjoB *o, uint32_t *lba,
+                                     int flag);
+
 #else /* Xorriso_findjob_on_expR */
 
 
@@ -434,6 +447,9 @@ int Findjob_set_found_path(struct FindjoB *o, char *path, int flag);
 
 int Findjob_get_found_path(struct FindjoB *o, char **path, int flag);
 
+int Findjob_get_last_data_file_block(struct FindjoB *o, uint32_t *lba,
+                                     int flag);
+
 #endif /* ! Xorriso_findjob_on_expR */
 
 
