@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -97,6 +97,8 @@ int Xorriso_create_empty_iso(struct XorrisO *xorriso, int flag)
  isoburn_ropt_set_input_charset(ropts, xorriso->in_charset);
  isoburn_ropt_set_data_cache(ropts, 1, 1, 0);
  isoburn_set_read_pacifier(drive, NULL, NULL);
+ isoburn_ropt_set_truncate_mode(ropts, 1, xorriso->file_name_limit);
+
  ret= isoburn_read_image(drive, ropts, &volset);
  Xorriso_process_msg_queues(xorriso,0);
  isoburn_ropt_destroy(&ropts, 0);
@@ -111,6 +113,8 @@ int Xorriso_create_empty_iso(struct XorrisO *xorriso, int flag)
  Xorriso_update_volid(xorriso, 0);
  xorriso->volset_change_pending= 0;
  xorriso->boot_count= 0;
+ xorriso->system_area_clear_loaded=
+                    (strcmp(xorriso->system_area_disk_path, "/dev/zero") == 0);
  xorriso->no_volset_present= 0;
  return(1);
 }
@@ -367,7 +371,7 @@ int Xorriso_get_volid(struct XorrisO *xorriso, char volid[33], int flag)
  bit1= do only report to fp
  bit2= is_default
  bit3= append -boot_image any next
- bit4= eventually concentrate boot options
+ bit4= concentrate boot options
 */
 int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
                              char *bin_path, int platform_id,
@@ -387,7 +391,7 @@ int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
  line= xorriso->result_line;
 
  if((flag & 16) && bin_path[0] != 0) {
-   /* Allow to concentrate boot options. */
+   /* Concentrate boot options. */
    memset(zeros, 0, 28);
    if(memcmp(id_string, zeros, 28) == 0 &&
       memcmp(selection_crit, zeros, 20) == 0)
@@ -562,12 +566,7 @@ int Xorriso_boot_status_non_mbr(struct XorrisO *xorriso, IsoImage *image,
    strcat(line, "\n");
    Xorriso_status_result(xorriso, filter, fp, flag & 2);
    return(0);
- }
- if(sa_type != 1 && sa_type != 2 && sa_type != 4 && sa_type != 5 &&
-    sa_type != 6)
-   return(0);
-
- if(sa_type == 1 || sa_type == 2) {
+ } else if(sa_type == 1 || sa_type == 2) {
    num_boots= iso_image_get_mips_boot_files(image, paths, 0);
    Xorriso_process_msg_queues(xorriso, 0);
    if(num_boots > 0) {
@@ -946,6 +945,7 @@ ex:;
 int Xorriso_get_volume(struct XorrisO *xorriso, IsoImage **volume,
                        int flag)
 {
+ *volume= NULL;
  if(xorriso->in_volset_handle==NULL) {
    if(flag & 1)
      return(0);
@@ -1090,7 +1090,8 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
       strncmp(dev, "mmc:", 4) != 0)
      sprintf(libburn_adr, "stdio:%s", dev);
    burn_preset_device_open(
-           xorriso->drives_exclusive && !(xorriso->mount_opts_flag & 1), 0, 0);
+           (xorriso->drives_exclusive && !(xorriso->mount_opts_flag & 1)) |
+           (xorriso->linux_scsi_dev_family << 2), 0, 0);
    aquire_flag= 1;
    if((xorriso->toc_emulation_flag & 2) && adr_mode == 3)
      aquire_flag|= 16;
@@ -1099,7 +1100,7 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
    if(xorriso->toc_emulation_flag & 8)
      aquire_flag|= 512;
    ret= isoburn_drive_aquire(&dinfo, libburn_adr, aquire_flag);
-   burn_preset_device_open(1, 0, 0);
+   burn_preset_device_open(1 | (xorriso->linux_scsi_dev_family << 2), 0, 0);
    Xorriso_process_msg_queues(xorriso,0);
    if(ret <= 0)
      {ret= 0; goto ex;}
@@ -1140,7 +1141,7 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
    burn_drive_info_free(dinfo);
    drive= NULL;
  } else if(give_up > 0 && !((flag & 1) || (xorriso->mount_opts_flag & 1))) {
-   Xorriso_give_up_drive(xorriso, give_up);
+   ret= Xorriso_give_up_drive(xorriso, give_up);
    if(ret <= 0)
      goto ex;
  }
@@ -1526,6 +1527,31 @@ struct elto_img_par {
 };
 
 
+/* @param ptype   0= unknown, 1= gpt-basdat, 2=gpt-hfsplus, 3=EFI
+   @param flag    bit0= isohybrid
+*/
+static int Xorriso_register_eltorito_gpt(struct XorrisO *xorriso,
+                                         struct elto_img_par *et_img,
+                                         int ptype,
+                                         int *efi_boot_part, int *first_efi,
+                                         int flag)
+{
+ if(flag & 1) {
+   if(ptype == 1 || ptype == 3)
+     et_img->do_gpt_basdat= 1;
+   else if(ptype == 2)
+     et_img->do_gpt_hfsplus= 1;
+   return(1);
+ } else if(*first_efi && et_img->platform_id == 0xef) {
+   *efi_boot_part= 1;
+   return(1);
+ }
+ if(et_img->platform_id == 0xef)
+   *first_efi= 0;
+ return(0);
+}
+
+
 /* @param ptype   0= unknown, 1= gpt-basdat, 2=gpt-hfsplus, 3=EFI
    @param flag    bit0= isohybrid
 */
@@ -1535,23 +1561,15 @@ static int Xorriso_search_eltorito_path(struct XorrisO *xorriso,
                                         int *found, int *efi_boot_part,
                                         int flag)
 {
- int first_efi= 1, et_idx;
+ int first_efi= 1, et_idx, ret;
 
  for(et_idx= 0; et_idx < elto_count; et_idx++) {
    if(strcmp(et_imgs[et_idx].path, path) != 0)
  continue;
-   if(flag & 1) {
-     if(ptype == 1 || ptype == 3)
-       et_imgs[et_idx].do_gpt_basdat= 1;
-     else if(ptype == 2)
-       et_imgs[et_idx].do_gpt_hfsplus= 1;
+   ret= Xorriso_register_eltorito_gpt(xorriso, et_imgs + et_idx,
+                                      ptype, efi_boot_part, &first_efi, flag);
+   if(ret > 0)
  break;
-   } else if(first_efi && et_imgs[et_idx].platform_id == 0xef) {
-     *efi_boot_part= 1;
- break;
-   }
-   if(et_imgs[et_idx].platform_id == 0xef)
-     first_efi= 0;
  }
  *found= et_idx;
  if(et_idx < elto_count)
@@ -1560,7 +1578,51 @@ static int Xorriso_search_eltorito_path(struct XorrisO *xorriso,
 }
 
 
+static int Xorriso_search_eltorito_lba(struct XorrisO *xorriso,
+                                       struct elto_img_par *et_imgs,
+                                       int elto_count,
+                                       unsigned int lba,
+                                       int *found, int flag)
+{
+ int et_idx;
+
+ for(et_idx= 0; et_idx < elto_count; et_idx++)
+   if(et_imgs[et_idx].lba == lba)
+ break;
+ *found= et_idx;
+ if(et_idx < elto_count)
+   return(1);
+ return(0);
+}
+
+
+int Xorriso_highest_data_block(struct XorrisO *xorriso, uint32_t *high_block,
+                               int flag)
+{
+ int ret;
+ struct FindjoB *job= NULL;
+ struct stat dir_stbuf;
+
+ *high_block= 0;
+ ret= Findjob_new(&job, "/", 0);
+ if(ret <= 0) {
+   Xorriso_no_findjob(xorriso, "[internal:last_data_file_block]", 0);
+   {ret= -1; goto ex;}
+ }
+ Findjob_set_action_type(job, 58, 0, 0);
+ ret= Xorriso_findi(xorriso, job, NULL,  (off_t) 0,
+                    NULL, "/", &dir_stbuf, 0, 0);
+ if(ret <= 0)
+   goto ex;
+ Findjob_get_last_data_file_block(job, high_block, 0);
+ex:;
+ Findjob_destroy(&job, 0);
+ return(ret);
+}
+
+
 /* @param flag bit0= do not record but only count
+               bit1= as_mkisofs
 */
 static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
                                      char **et_lines, int et_line_count,
@@ -1568,35 +1630,45 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
                                      char **cmds, int *cmd_count, int flag)
 {
  int ret= 0, i, num_count, mkisofs, line_count, idx, et_idx, isohybrid= 0;
- int ptype, gpt_idx;
- int efi_boot_part= 0, full_sparc_part= 0, have_sparc_part= 0;
+ int ptype, gpt_idx, j, pad, mbr_idx;
+ int efi_boot_part= 0, full_sparc_part= 0, have_sparc_part= 0, fe_dummy= 1;
  int appended_as_gpt= 0, have_prep= 0, did_sysarea= 0, cared_for_apm= 0;
  int cared_for_sparc= 0, have_hfsplus= 0;
  int have_sysarea= 0, ptable_killer, imported_iso, have_alpha_ldr_path= 0;
+ int have_protective_msdos= 0, appended_partition= 0, part_like_isohybrid= 0;
  unsigned long int sa_options= 0, partno, id_tag, perms, start_cyl, num_blocks;
  unsigned long int part_status, part_type, start_block;
+ uint32_t high_block= 0;
  char name[24], *textpt, *contentpt, *buf= NULL;
  char **lines= NULL;
  double num[8];
  char *cat_path= "";
  struct elto_img_par *et_imgs= NULL;
  int elto_count= 0;
- uint32_t img_blocks= 0;
+ uint32_t img_blocks= 0, mbr_parts_end= 0;
  struct FindjoB *job= NULL;
  struct stat dir_stbuf;
  IsoImage *image;
  char *volid, *crt, *mdt, *ext, *eft, uuid[17];
+ char **app_pseudo_paths= NULL;
 
  struct mbr_par {
    uint8_t ptype;
+   uint64_t start_block;
+   uint64_t block_count;
+   int appended;
+   int has_path;
  };
  struct mbr_par *mbrpts= NULL;
  int mbr_count= 0;
 
  struct gpt_par {
    int ptype; /* 0= unknown, 1= gpt-basdat, 2=gpt-hfsplus, 3=EFI */
+   int is_gap;
    int has_path;
    char *path;
+   uint64_t start_block;
+   uint64_t block_count;
  };
  struct gpt_par *gpts= NULL;
  int gpt_count= 0;
@@ -1666,10 +1738,25 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
    }
  }
 
+ ret= Xorriso_highest_data_block(xorriso, &high_block, 0);
+ if(ret < 0)
+   goto ex;
+ if(ret == 0)
+   high_block = img_blocks - 1;
+
  if(elto_count > 0) {
    Xorriso_alloc_meM(et_imgs, struct elto_img_par, elto_count);
-   for(et_idx= 0; et_idx < elto_count; et_idx++)
+   for(et_idx= 0; et_idx < elto_count; et_idx++) {
      et_imgs[et_idx].path= NULL;
+     et_imgs[et_idx].ldsiz= -1;
+   }
+   Xorriso_alloc_meM(app_pseudo_paths, char *, elto_count);
+   for(i= 0; i < elto_count; i++)
+     app_pseudo_paths[i]= NULL;
+   for(i= 0; i < elto_count; i++) {
+     Xorriso_alloc_meM(app_pseudo_paths[i], char, 80);
+     app_pseudo_paths[i][0]= 0;
+   }
  }
  if(mbr_count > 0)
    Xorriso_alloc_meM(mbrpts, struct mbr_par, mbr_count);
@@ -1704,7 +1791,19 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
      sprintf(buf, "--modification-date=");
    else
      sprintf(buf, "-volume_date uuid ");
-   strncpy(uuid, crt, 16);
+   pad= 0;
+   for(j= 0; j < 16; j++) {
+     if(pad) {
+       uuid[j]= '0';
+     } else if(crt[j] == 0) {
+       pad= 1;
+       uuid[j]= '0';
+     } else if(crt[j] < '0' || crt[j] > '9') {
+       uuid[j]= '0';
+     } else {
+       uuid[j]= crt[j];
+     }
+   }
    uuid[16]= 0;
    Text_shellsafe(uuid, buf, 1);
  }
@@ -1771,11 +1870,23 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
      idx= num[0] - 1;
      et_imgs[idx].sel_crit= textpt;
 
+   } else if(strcmp(name, "System area summary:") == 0) {
+     if(strstr(textpt, "protective-msdos-label") != NULL)
+       have_protective_msdos= 1;
+
    } else if(strcmp(name, "MBR partition      :") == 0) {
      sscanf(contentpt, "%lu 0x%lx 0x%lx %lu %lu",
             &partno, &part_status, &part_type, &start_block, &num_blocks);
      idx= partno - 1;
      mbrpts[idx].ptype= part_type;
+     mbrpts[idx].start_block= start_block;
+     mbrpts[idx].block_count= num_blocks;
+     if(num_blocks > 0 && start_block + num_blocks > mbr_parts_end)
+       mbr_parts_end= start_block + num_blocks; 
+
+   } else if(strcmp(name, "MBR partition path :") == 0) {
+     idx= num[0] - 1;
+     mbrpts[idx].has_path= 1;
 
    } else if(strcmp(name, "GPT type GUID      :") == 0) {
      idx= num[0] - 1;
@@ -1792,14 +1903,19 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
      idx= num[0] - 1;
      if(img_blocks <= num[1] && num[2] > 0)
        appended_as_gpt= 1;
-
-     /* >>> ??? can there be independent partitions inside the image ? */;
+     gpts[idx].start_block= num[1];
+     gpts[idx].block_count= num[2];
 
    } else if(strcmp(name, "GPT partition path :") == 0) {
      idx= num[0] - 1;
      gpts[idx].has_path= 1;
      gpts[idx].path= textpt;
 
+   } else if(strcmp(name, "GPT partition name :") == 0) {
+     idx= num[0] - 1;
+     if(strstr(contentpt, " 470061007000") != NULL) /* "Gap"... */
+       gpts[idx].is_gap= 1;
+
    } else if(strcmp(name, "APM partition name :") == 0) {
      idx= num[0] - 1;
      if(strcmp(textpt, "HFSPLUS_Hybrid") == 0)
@@ -1820,6 +1936,9 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
    }
  }
 
+ if(!have_protective_msdos)
+   appended_as_gpt= 0;
+
  /* Second pass: scan for System Area info */
  for(i= 0; i < line_count; i++) {
    buf[0]= 0;
@@ -1907,12 +2026,18 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
    } else if(strcmp(name, "MBR partition      :") == 0) {
      sscanf(contentpt, "%lu 0x%lx 0x%lx %lu %lu",
             &partno, &part_status, &part_type, &start_block, &num_blocks);
-     if(img_blocks <= start_block && num_blocks > 0) {
+     if(num_blocks > 0 && part_type != 0x00 && part_type != 0xee &&
+        (img_blocks <= start_block ||
+         (have_protective_msdos && img_blocks == mbr_parts_end &&
+          partno > 1))) {
        if(!appended_as_gpt) {
          sprintf(buf, "-append_partition %lu 0x%lx ", partno, part_type);
          Xorriso_add_intvl_adr(xorriso, buf, (uint64_t) start_block,
                                ((uint64_t) start_block) + num_blocks - 1, "d",
                                imported_iso);
+         if(partno >= 1 && (int) partno <= mbr_count)
+           mbrpts[partno - 1].appended= 1;
+         appended_partition= 1;
        }
      } else if(part_type == 0x41 && have_prep) {
        if(mkisofs) {
@@ -1923,8 +2048,13 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
        Xorriso_add_intvl_adr(xorriso, buf, (uint64_t) start_block,
                              ((uint64_t) start_block) + num_blocks - 1, "d",
                              imported_iso);
+    } else if((part_status & 0x80) && part_type == 0x00 &&
+              start_block == 0 && num_blocks == 1) {
+       if(mkisofs)
+         sprintf(buf, "--mbr-force-bootable");
+       else
+         sprintf(buf, "-boot_image any mbr_force_bootable=on");
      }
-
    } else if(strcmp(name, "MBR partition path :") == 0) {
      idx= num[0] - 1;
      if(mbrpts[idx].ptype == 0x41) {
@@ -1996,19 +2126,49 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
        part_type= 0xef;
      else
        part_type= 0xcd;
-     if(img_blocks <= num[1] && num[2] > 0) {
-       if(appended_as_gpt == 1) {
-         if(mkisofs)
-           sprintf(buf, "-appended_part_as_gpt");
-         else
-           sprintf(buf, "-boot_image any appended_part_as=gpt");
-         Xorriso_record_cmd_linE
-         appended_as_gpt= 2;
+
+     if(high_block < num[1] && num[2] > 0 && !gpts[idx].is_gap) {
+       for(mbr_idx = 0; mbr_idx < mbr_count; mbr_idx++) {
+         if(mbrpts[mbr_idx].start_block == num[1] &&
+            mbrpts[mbr_idx].block_count == num[2])
+       break;
+       }
+       if(mbr_idx >= mbr_count) {
+         if(appended_as_gpt == 1) {
+           if(mkisofs)
+             sprintf(buf, "-appended_part_as_gpt");
+           else
+             sprintf(buf, "-boot_image any appended_part_as=gpt");
+           Xorriso_record_cmd_linE
+           appended_as_gpt= 2;
+         }
+         sprintf(buf, "-append_partition %d 0x%lx ", idx + 1, part_type);
+         Xorriso_add_intvl_adr(xorriso, buf, (uint64_t) num[1],
+                               (uint64_t) (num[1] + num[2] - 1.0), "d",
+                               imported_iso);
+         appended_partition= 1;
+       }
+
+       ret= Xorriso_search_eltorito_lba(xorriso, et_imgs, elto_count,
+                                        (unsigned int) (num[1] / 4.0),
+                                        &et_idx, 0);
+       if(ret > 0) {
+         if(!(et_imgs[et_idx].do_gpt_basdat ||
+              et_imgs[et_idx].do_gpt_hfsplus ||
+              part_like_isohybrid)) {
+           if(mkisofs)
+             sprintf(buf, "-part_like_isohybrid");
+           else
+             sprintf(buf, "-boot_image any part_like_isohybrid=on");
+           Xorriso_record_cmd_linE
+           buf[0]= 0;
+           part_like_isohybrid= 1;
+         }
+         /*  mark el torito for  -isohybrid-gpt-... */
+         Xorriso_register_eltorito_gpt(xorriso, et_imgs + et_idx,
+                                       gpts[idx].ptype, &efi_boot_part,
+                                       &fe_dummy, 1);
        }
-       sprintf(buf, "-append_partition %d 0x%lx ", idx + 1, part_type);
-       Xorriso_add_intvl_adr(xorriso, buf, (uint64_t) num[1],
-                             (uint64_t) (num[1] + num[2] - 1.0), "d",
-                             imported_iso);
 
      } else if(gpts[idx].ptype == 3 && gpts[idx].has_path == 0 &&
                img_blocks >= num[1] + num[2] && !efi_boot_part) {
@@ -2038,9 +2198,10 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
      /* Check El Torito EFI boot images for same path */
      for(et_idx= 0; isohybrid && et_idx < elto_count; et_idx++)
        if(strcmp(et_imgs[et_idx].path, textpt) == 0) {
-         if(apms[idx].ptype == 1)
+         if(apms[idx].ptype == 1) {
            et_imgs[et_idx].do_apm_hfsplus= 1;
            cared_for_apm= 1;
+         }
      break;
        }
 
@@ -2119,8 +2280,9 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
      start_cyl= 0xffffffff;
      sscanf(contentpt, "%lu 0x%lx 0x%lx %lu %lu",
             &partno, &id_tag, &perms, &start_cyl, &num_blocks);
-     if(partno > 0 && partno < 9 && start_cyl == 0 && num_blocks == img_blocks
-        && ((partno == 1 && id_tag == 4) || (partno > 1 && id_tag == 2)))
+     if(partno > 0 && partno < 9 && start_cyl == 0 && 
+        num_blocks >= img_blocks - 600 && num_blocks <= img_blocks &&
+        ((partno == 1 && id_tag == 4) || (partno > 1 && id_tag == 2)))
        full_sparc_part|= (1 << (partno - 1));
 
    } else if(strcmp(name, "PALO header version:") == 0) {
@@ -2265,6 +2427,68 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
        sprintf(buf, "-boot_image any next");
      Xorriso_record_cmd_linE
    }
+   if(et_imgs[idx].path[0] == 0) {
+
+     /* >>> need way to eploit El Torito img blks : */;
+
+     /* Check whether appended partition */;
+     for(i= 0; i < mbr_count; i++)
+       if((mbrpts[i].appended || !mbrpts[i].has_path) &&
+          mbrpts[i].start_block == ((uint64_t) et_imgs[idx].lba) * 4 &&
+          (mbrpts[i].block_count == (uint64_t) et_imgs[idx].ldsiz ||
+           et_imgs[idx].ldsiz == 0 || et_imgs[idx].ldsiz == 1)) 
+     break;
+     if (i < mbr_count) {
+       if(!mbrpts[i].appended) {
+         mbrpts[i].appended= 1;
+         if(!appended_as_gpt) {
+           sprintf(buf, "-append_partition %lu 0x%lx ", (unsigned long) i + 1,
+                         (unsigned long) mbrpts[i].ptype);
+           Xorriso_add_intvl_adr(xorriso, buf,
+                                 (uint64_t) mbrpts[i].start_block,
+                                 ((uint64_t) mbrpts[i].start_block) +
+                                 mbrpts[i].block_count - 1,
+                                 "d", imported_iso);
+           Xorriso_record_cmd_linE
+           appended_partition= 1;
+           buf[0]= 0;
+         }
+       }
+       sprintf(app_pseudo_paths[idx],
+               "--interval:appended_partition_%d_start_%lus_size_%lud:all::",
+               i + 1,
+               (unsigned long) et_imgs[idx].lba,
+               (unsigned long) mbrpts[i].block_count);
+       et_imgs[idx].path= app_pseudo_paths[idx];
+     }
+     if (et_imgs[idx].path[0] == 0) {
+       for(i= 0; i < gpt_count; i++) {
+         if(have_protective_msdos && (
+            gpts[i].start_block == ((uint64_t) et_imgs[idx].lba) * 4 &&
+            (gpts[i].block_count == (uint64_t) et_imgs[idx].ldsiz ||
+            et_imgs[idx].ldsiz == 0 || et_imgs[idx].ldsiz == 1)))
+       break;
+       }
+       if (i < gpt_count) {
+         sprintf(app_pseudo_paths[idx],
+                 "--interval:appended_partition_%d_start_%lus_size_%lud:all::",
+                 i + 1,
+                 (unsigned long) et_imgs[idx].lba,
+                 (unsigned long) gpts[i].block_count);
+         et_imgs[idx].path= app_pseudo_paths[idx];
+       }
+     }
+     if (et_imgs[idx].path[0] == 0) {
+       if(!(flag & 1)) {
+         sprintf(xorriso->info_text,
+              "Cannot enable EL Torito boot image #%d because it is not a data file in the ISO filesystem",
+              idx + 1);
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+       }
+       buf[0]= 0;
+ continue;
+     }
+   }
    if(et_imgs[idx].platform_id == 0) {
      if(mkisofs)
        sprintf(buf, "-b ");
@@ -2284,19 +2508,6 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
  continue;
      }
    }
-   if(et_imgs[idx].path[0] == 0) {
-
-     /* >>> need way to eploit El Torito img blks : */;
-
-     if(!(flag & 1)) {
-       sprintf(xorriso->info_text,
-              "Cannot enable EL Torito boot image #%d because it is not a data file in the ISO filesystem",
-              idx + 1);
-       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
-     }
-     buf[0]= 0;
- continue;
-   }
    Text_shellsafe(et_imgs[idx].path, buf, 1);
    Xorriso_record_cmd_linE
    if(!mkisofs) {
@@ -2322,7 +2533,7 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
    }
    if(buf[0])
      Xorriso_record_cmd_linE
-   if(et_imgs[idx].ldsiz > 0) {
+   if(et_imgs[idx].ldsiz >= 0) {
      if(mkisofs)
        sprintf(buf, "-boot-load-size %d", et_imgs[idx].ldsiz);
      else
@@ -2389,6 +2600,14 @@ after_el_torito:
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
  }
 
+ if(appended_partition) {
+   if(mkisofs)
+     sprintf(buf, "-no-pad");
+   else
+     sprintf(buf, "-padding 0");
+   Xorriso_record_cmd_linE
+ }
+
  ret= 1;
 ex:
  xorriso->show_hfs_cmds= NULL;
@@ -2396,6 +2615,12 @@ ex:
  Xorriso_free_meM(apms);
  Xorriso_free_meM(gpts);
  Xorriso_free_meM(mbrpts);
+ if(app_pseudo_paths != NULL) {
+   for(i= 0; i < elto_count; i++)
+     if(app_pseudo_paths[i] != NULL)
+       Xorriso_free_meM(app_pseudo_paths[i]);
+   Xorriso_free_meM(app_pseudo_paths);
+ }
  Xorriso_free_meM(et_imgs);
  Xorriso_free_meM(lines);
  Xorriso_free_meM(buf);
@@ -2408,7 +2633,7 @@ ex:
 
 /* @param flag bit0= currently not significant:
                      report is about El Torito rather than System Area
-               bit1= report -as mkisofsoptions
+               bit1= report -as mkisofs options
                bit15= dispose cmds
 */
 static int Xorriso_report_to_cmd(struct XorrisO *xorriso,
@@ -2472,12 +2697,12 @@ static void Xorriso_report_lines(struct XorrisO *xorriso,
 
 
 /* @param flag bit0= report El Torito rather than System Area
-               >>> bit1= with form "cmd" do not report but rather execute
+               bit1= with form "cmd" do not report but rather execute
 */
 int Xorriso_report_system_area(struct XorrisO *xorriso, char *form, int flag)
 {
- int ret, line_count, cmd_count, et_line_count= 0, sa_line_count= 0;
- int do_cmd= 0, as_mkisofs= 0;
+ int ret, line_count, cmd_count= 0, et_line_count= 0, sa_line_count= 0;
+ int do_cmd= 0, as_mkisofs= 0, i;
  char **lines = NULL, **et_lines= NULL, **sa_lines= NULL, **cmds= NULL;
  IsoImage *image;
 
@@ -2566,9 +2791,26 @@ int Xorriso_report_system_area(struct XorrisO *xorriso, char *form, int flag)
    }
  }
  if(flag & 2) {
-
-   /* >>> execute commands */;
-
+   if(cmd_count > 0) {
+     ret= Xorriso_execute_option(xorriso,
+               "-boot_image any discard -boot_image any system_area=/dev/zero",
+               1 | 16);
+     if(ret <= 0)
+       goto ex;
+     for(i= 0; i < cmd_count; i++) {
+       ret= Xorriso_execute_option(xorriso, cmds[i], 1 | 16);
+       if(ret <= 0)
+         goto ex;
+     }
+     sprintf(xorriso->info_text,
+             "Replayed %d boot related commands", cmd_count);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
+   } else {
+     Xorriso_msgs_submit(xorriso, 0,
+                         "No proposals available for boot related commands",
+                         0, "NOTE", 0);
+     ret= 2; goto ex;
+   }
  } else if(do_cmd) {
    Xorriso_report_lines(xorriso, cmds, cmd_count);
  } else {
