@@ -1108,9 +1108,8 @@ int Hex_to_bin(char *hex,
 
 #ifndef Xorriso_fileliste_externaL 
 
-/* ??? ts A71006 : Is this compatible with mkisofs pathspecs ?
-                   I dimly remember so */
-
+/* @param flag bit0= pathspec mode "on" rather than "as_mkisofs"
+*/
 int Fileliste__target_source_limit(char *line, char sep, char **limit_pt,
                                     int flag)
 {
@@ -1118,7 +1117,7 @@ int Fileliste__target_source_limit(char *line, char sep, char **limit_pt,
 
  for(npt= line;*npt!=0;npt++) {
    if(*npt=='\\') {
-     if(*(npt+1)!=0)
+     if(*(npt + 1) == '=' || (*(npt + 1) == '\\' && !(flag & 1)))
        npt++;
  continue;
    }
@@ -1203,11 +1202,12 @@ int Xorriso__bourne_to_reg(char bourne_expr[], char reg_expr[], int flag)
         *(wpt++)= '!';
       }
     break;case '^':
-      if(in_square_brackets)
+      if(in_square_brackets) {
         *(wpt++)= '^';
-      else
+      } else {
         *(wpt++)= '\\';
         *(wpt++)= '^';
+      }
     break;case '$':
       *(wpt++)= '\\';
       *(wpt++)= '$';
