@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -95,18 +95,37 @@ int Xorriso_option_acl(struct XorrisO *xorriso, char *mode, int flag)
 }
 
 
+/* @param flag bit3= unescape \\
+*/
+static void unescape_pathspec_part(char *rpt, int flag)
+{
+ char *wpt;
+
+ wpt= rpt;
+ for(; *rpt != 0; rpt++) {
+   if(*rpt == '\\') {
+     if(*(rpt + 1) == '=')
+ continue;
+     if((flag & 8) && *(rpt + 1) == '\\')
+       rpt++;
+   }
+   *(wpt++)= *rpt;
+ }
+ *wpt= 0;
+}
+
+
 /* Option -add */
 /* @param flag bit0=do not report the added item
                bit1=do not reset pacifier, no final pacifier message
                bit2= prepend ISO working directory in any case
-               bit3= unescape \\
 */
 int Xorriso_option_add(struct XorrisO *xorriso, int argc, char **argv,
                        int *idx, int flag)
 {
- int i, end_idx, ret, was_failure= 0, fret, optc= 0, split;
+ int i, end_idx, ret, was_failure= 0, fret, optc= 0, split, as_mkisofs= 0;
  char *target= NULL, *source= NULL, *ept, *eff_path= NULL;
- char **optv= NULL, *rpt, *wpt;
+ char **optv= NULL;
 
  ret= Xorriso_opt_args(xorriso, "-add", argc, argv, *idx, &end_idx,
                        &optc, &optv, ((!!xorriso->allow_graft_points)<<2)|2);
@@ -117,6 +136,8 @@ int Xorriso_option_add(struct XorrisO *xorriso, int argc, char **argv,
  Xorriso_alloc_meM(source, char, SfileadrL);
  Xorriso_alloc_meM(eff_path, char, SfileadrL);
 
+ if(xorriso->allow_graft_points & 2)
+   as_mkisofs= 8;
  if(!(flag&2))
    Xorriso_pacifier_reset(xorriso, 0);
  for(i= 0; i<optc; i++) {
@@ -125,27 +146,21 @@ int Xorriso_option_add(struct XorrisO *xorriso, int argc, char **argv,
    strcpy(source, optv[i]);
    split= 0;
    if(xorriso->allow_graft_points) {
-     ret= Fileliste__target_source_limit(target, '=', &ept, 0);
+     ret= Fileliste__target_source_limit(target, '=', &ept,
+                                         !(xorriso->allow_graft_points & 2));
      if(ret>0) {
        *ept= 0;
        strcpy(source, ept+1);
        split= 1;
      }
      /* unescape \= */;
-     if(split)
-       rpt= wpt= target;
-     else
-       rpt= wpt= source;
-     for(; *rpt!=0; rpt++) {
-       if(*rpt=='\\') {
-         if(*(rpt+1)=='=')
-     continue;
-         if((flag & 8) && *(rpt + 1) == '\\')
-           rpt++;
-       }
-       *(wpt++)= *rpt;
+     if(split) {
+       unescape_pathspec_part(target, as_mkisofs);
+       if(as_mkisofs)
+         unescape_pathspec_part(source, as_mkisofs);
+     } else {
+       unescape_pathspec_part(source, as_mkisofs);
      }
-     *wpt= 0;
    }
    if(split==0)
      strcpy(target, source);
@@ -652,7 +667,7 @@ unusable_size:;
    ret= Xorriso_blank_media(xorriso, mode_flag&3);
  if(ret==0)
    return(ret);
- if(ret <= 0) { /* in case of success, above functions will have re-aquired */
+ if(ret <= 0) { /* in case of success, above functions will have re-acquired */
    aq_ret= Xorriso_reaquire_outdev(xorriso, 0); /* actually give up drive */
    if(ret<aq_ret)
      return(ret);
@@ -714,6 +729,11 @@ treatment_patch:;
    else
      xorriso->patch_system_area= 0;
 
+ } else if(strcmp(treatpt, "replay")==0) {
+   ret= Xorriso_report_system_area(xorriso, "cmd", 2);
+   if(ret <= 0)
+     goto ex;
+
  } else if(strcmp(treatpt, "discard")==0) {
    xorriso->keep_boot_image= 0;
    xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~0x3ff) | 0;
@@ -812,10 +832,14 @@ treatment_patch:;
    xorriso->boot_image_bin_path[0] = 0;
    xorriso->boot_efi_default= 0;
  } else if(strncmp(treatpt, "bin_path=", 9) == 0) {
-   ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, treatpt + 9,
-                                   xorriso->boot_image_bin_path, 2);
-   if(ret <= 0)
-     goto ex;
+   if(strncmp(treatpt + 9,  "--interval:appended_partition_", 30) == 0) {
+     strcpy(xorriso->boot_image_bin_path, treatpt + 9);
+   } else {
+     ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, treatpt + 9,
+                                     xorriso->boot_image_bin_path, 2);
+     if(ret <= 0)
+       goto ex;
+   }
    xorriso->keep_boot_image= 0;
    if(isolinux_grub) {
      xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 1;
@@ -830,10 +854,14 @@ treatment_patch:;
    xorriso->boot_image_bin_path[0] = 0;
    xorriso->boot_efi_default= 0;
  } else if(strncmp(treatpt, "efi_path=", 9) == 0) {
-   ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, treatpt + 9,
+   if(strncmp(treatpt + 9,  "--interval:appended_partition_", 30) == 0) {
+     strcpy(xorriso->boot_image_bin_path, treatpt + 9);
+   } else {
+     ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, treatpt + 9,
                                    xorriso->boot_image_bin_path, 2);
-   if(ret <= 0)
-     goto ex;
+     if(ret <= 0)
+       goto ex;
+   }
    xorriso->keep_boot_image= 0;
    xorriso->boot_efi_default= 1;
 
@@ -910,6 +938,8 @@ treatment_patch:;
    strncpy(parm, treatpt + 5, parm_len);
    parm[parm_len]= 0;
    ret= Xorriso_set_hppa_boot_parm(xorriso, eqpt + 1, parm, 0);
+   if(ret <= 0)
+     goto ex;
 
  } else if(strncmp(treatpt, "alpha_boot=", 11) == 0) {
    sprintf(eff_path, "-boot_image %s %s", formpt, treatpt);
@@ -917,6 +947,8 @@ treatment_patch:;
    if(ret <= 0)
      goto ex;
    ret = Xorriso_set_alpha_boot(xorriso, treatpt + 11, 0);
+   if(ret <= 0)
+     goto ex;
 
  } else if(strncmp(treatpt, "boot_info_table=", 16)==0) {
    if(strcmp(treatpt + 16, "off") == 0)
@@ -982,6 +1014,8 @@ treatment_patch:;
    if(ret <= 0)
      goto ex;
    xorriso->system_area_options&= ~0x4000;
+   if(strcmp(treatpt + 12, "/dev/zero") == 0)
+     xorriso->system_area_clear_loaded= 1;
 
  } else if(strncmp(treatpt, "partition_table=", 16)==0) {
    if(strcmp(treatpt + 16, "off") == 0) {
@@ -1034,9 +1068,20 @@ treatment_patch:;
      xorriso->appended_as_gpt = 1;
    } else if(strcmp(treatpt + 17, "mbr") == 0) {
      xorriso->appended_as_gpt = 0;
+     xorriso->appended_as_apm = 0;
+   } else if(strcmp(treatpt + 17, "apm") == 0) {
+     xorriso->appended_as_apm = 1;
    } else
      was_ok= 0;
 
+ } else if(strncmp(treatpt, "part_like_isohybrid=", 20) == 0) {
+   if(strcmp(treatpt + 20, "on") == 0)
+     xorriso->part_like_isohybrid= 1;
+   else if(strcmp(treatpt + 20, "off") == 0)
+     xorriso->part_like_isohybrid= 0;
+   else
+     was_ok= 0;
+
  } else if(strncmp(treatpt, "partition_hd_cyl=", 17)==0) {
    u= 0;
    sscanf(treatpt + 17, "%u", &u);
@@ -1234,6 +1279,19 @@ treatment_patch:;
    else 
      xorriso->system_area_options|= 0x4000;
 
+ } else if(strncmp(treatpt, "mbr_force_bootable=", 19) == 0) {
+   if(strcmp(treatpt + 19, "off") == 0) {
+     xorriso->system_area_options&= ~(1 << 15);
+   } else if(strcmp(treatpt + 19, "on") == 0) {
+     xorriso->system_area_options|= (1 << 15);
+   } else {
+     sprintf(xorriso->info_text,
+             "-boot_image %s mbr_force_bootable=: unknown mode : %s",
+             formpt, treatpt + 19);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+     {ret= 0; goto ex;}
+   }
+
  } else
    was_ok= 0;
 
@@ -1275,7 +1333,7 @@ int Xorriso_option_calm_drive(struct XorrisO *xorriso, char *which, int flag)
 /* Option -cd alias -cdi */
 int Xorriso_option_cdi(struct XorrisO *xorriso, char *iso_rr_path, int flag)
 {
- char *path= NULL, *eff_path= NULL;
+ char *path= NULL, *eff_path= NULL, *namept;
  int ret;
 
  Xorriso_alloc_meM(path, char, SfileadrL);
@@ -1324,7 +1382,8 @@ int Xorriso_option_cdi(struct XorrisO *xorriso, char *iso_rr_path, int flag)
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    {ret= 0; goto ex;}
  }
- strcpy(xorriso->wdi, eff_path);
+ Xorriso_truncate_path_comps(xorriso, eff_path, path, &namept, 0);
+ strcpy(xorriso->wdi, namept);
 
  Xorriso_option_pwdi(xorriso, 0);
  ret= 1;
@@ -2033,7 +2092,7 @@ int Xorriso_option_close_filter_list(struct XorrisO *xorriso, int flag)
 
 
 /* Option -commit */
-/* @param flag bit0= leave indrive and outdrive aquired as they were,
+/* @param flag bit0= leave indrive and outdrive acquired as they were,
                      i.e. do not aquire outdrive as new in-out-drive
                bit1= do not perform eventual -reassure
    @return <=0 error , 1 success, 2 revoked by -reassure , 3 no change pending
@@ -2229,6 +2288,7 @@ int Xorriso_option_concat(struct XorrisO *xorriso,
 {
  int ret, end_idx, optc= 0, progc= 0, iso_rr_start, prog_end_idx= -1;
  char **optv= NULL, **progv= NULL, *delimiter_mem= NULL;
+ char *delimiter= NULL;
 
  /* Must be done before any goto ex; */
  end_idx= Xorriso_end_idx(xorriso, argc, argv, *idx, 1);
@@ -2254,9 +2314,16 @@ int Xorriso_option_concat(struct XorrisO *xorriso,
      ret= 0; goto ex;
    }
 
+   ret= Xorriso_check_thing_len(xorriso, argv[*idx + 1],
+                                sizeof(xorriso->list_delimiter), "-concat",
+                                "Delimiter", 0);
+   if(ret <= 0)
+     goto ex;
    Xorriso_alloc_meM(delimiter_mem, char, strlen(xorriso->list_delimiter) + 1);
+   Xorriso_alloc_meM(delimiter, char, strlen(argv[*idx + 1]) + 1);
    strcpy(delimiter_mem, xorriso->list_delimiter);
-   strcpy(xorriso->list_delimiter, argv[*idx + 1]);
+   strcpy(delimiter, argv[*idx + 1]);
+   strcpy(xorriso->list_delimiter, delimiter);
    ret= Xorriso_opt_args(xorriso, "-concat pipe", argc , argv, *idx + 2,
                          &prog_end_idx, &progc, &progv, 4 | 128);
    strcpy(xorriso->list_delimiter, delimiter_mem);
@@ -2282,15 +2349,16 @@ int Xorriso_option_concat(struct XorrisO *xorriso,
                      progc, progv, optc, optv, 0);
 ex:;
  if(progv != NULL) {
-   if(delimiter_mem != NULL)
-      strcpy(xorriso->list_delimiter, argv[*idx + 2]);
+   if(delimiter_mem != NULL && delimiter != NULL)
+     strcpy(xorriso->list_delimiter, delimiter);
    Xorriso_opt_args(xorriso, "-concat", argc, argv, *idx + 2, &prog_end_idx,
                     &progc, &progv, 256);
-   if(delimiter_mem != NULL)
+   if(delimiter_mem != NULL && delimiter != NULL)
      strcpy(xorriso->list_delimiter, delimiter_mem);
  }
  Xorriso_opt_args(xorriso, "-concat", argc, argv, iso_rr_start, &end_idx,
                   &optc, &optv, 256);
+ Xorriso_free_meM(delimiter);
  Xorriso_free_meM(delimiter_mem);
  *idx= end_idx;
  return(ret); 
