@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -65,7 +65,7 @@ int Xorriso_option_data_cache_size(struct XorrisO *xorriso, char *num_tiles,
                 bit3= regard overwriteable media as blank
                 bit4= if the drive is a regular disk file: truncate it to
                       the write start address
-                bit5= do not print toc of aquired drive
+                bit5= do not print toc of acquired drive
                 bit6= do not calm down drive after aquiring it
     @return <=0 error , 1 success, 2 revoked by -reassure
 */
@@ -346,6 +346,26 @@ int Xorriso_option_early_stdio_test(struct XorrisO *xorriso, char *mode,
 }
 
 
+/* Command -ecma119_map */
+int Xorriso_option_ecma119_map(struct XorrisO *xorriso, char *mode, int flag)
+{
+ if(strcmp(mode, "unmapped") == 0)
+   xorriso->ecma119_map= 0;
+ else if(strcmp(mode, "stripped") == 0)
+   xorriso->ecma119_map= 1;
+ else if(strcmp(mode, "uppercase") == 0)
+   xorriso->ecma119_map= 2;
+ else if(strcmp(mode, "lowercase") == 0)
+   xorriso->ecma119_map= 3;
+ else {
+   sprintf(xorriso->info_text, "-ecma119_map: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -eject */
 /* @param flag bit0=do not report toc of eventually remaining drives
 */
@@ -628,6 +648,29 @@ int Xorriso_option_extract_cut(struct XorrisO *xorriso, char *iso_rr_path,
 }
 
 
+/* Command -file_name_limit */
+int Xorriso_option_file_name_limit(struct XorrisO *xorriso, char *value,
+                                   int flag)
+{
+ int ret, sub_flag= 0;
+ double num;
+
+ if(value[0] == '+')
+   sub_flag|= 1;
+ num= Scanf_io_size(value + (sub_flag & 1), 0);
+ if(num < 64 || num > 255) {
+   sprintf(xorriso->info_text,
+           "-file_name_limit: Value '%s' out of range [64..255]", value);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   return(0);
+ }
+ if(num == xorriso->file_name_limit)
+   return(1);
+ ret= Xorriso_set_file_name_limit(xorriso, (int) num, sub_flag);
+ return(ret > 0);
+}
+
+
 /* Option -file_size_limit */
 int Xorriso_option_file_size_limit(struct XorrisO *xorriso,
                                    int argc, char **argv, int *idx, int flag)
@@ -682,6 +725,23 @@ static int Xorriso_determine_name_space(struct XorrisO *xorriso,
  return(0);
 }
 
+static int Xorriso_truncate_const_find_name(struct XorrisO *xorriso,
+                                            char *expr, char *buffer,
+                                            char **namept, int flag)
+{
+ int ret;
+
+ *namept= expr;
+ ret= Xorriso_truncate_path_comps(xorriso, expr, buffer, namept, 1);
+ if(ret < 0) {
+   sprintf(xorriso->info_text,
+           "-find[ix]: cannot truncate constant -name to -file_name_limit: ");
+   Text_shellsafe(expr, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1);
+}
 
 /* Option -find alias -findi, and -findx */
 /* @param flag bit0= -findx rather than -findi
@@ -700,7 +760,7 @@ int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
  struct FindjoB *job, *first_job= NULL, *new_job;
  char *start_path, *path= NULL, *cpt, *other_path_start= NULL, *cd_pt;
  char *access_acl_text= NULL, *default_acl_text= NULL, *list_extattr_mode;
- char *arg1_pt;
+ char *arg1_pt, *namept;
 
  struct stat dir_stbuf;
  uid_t user= 0;
@@ -736,7 +796,10 @@ not_enough_arguments:;
        goto sorry_ex;
      }
      i++;
-     ret= Findjob_set_name_expr(job, argv[i], 0);
+     ret= Xorriso_truncate_const_find_name(xorriso, argv[i], path, &namept, 0);
+     if(ret <= 0)
+       goto ex;
+     ret= Findjob_set_name_expr(job, namept, 0);
      if(ret<=0) {
        sprintf(xorriso->info_text, "-find[ix]: cannot set -name expression ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
@@ -746,7 +809,10 @@ not_enough_arguments:;
      if(i+1>=end_idx)
        goto not_enough_arguments;
      i++;
-     ret= Findjob_set_name_expr(job, argv[i], 1);
+     ret= Xorriso_truncate_const_find_name(xorriso, argv[i], path, &namept, 0);
+     if(ret <= 0)
+       goto ex;
+     ret= Findjob_set_name_expr(job, namept, 1);
      if(ret<=0) {
        sprintf(xorriso->info_text,
                "-find[ix]: cannot set -wholename expression ");
@@ -945,6 +1011,19 @@ not_enough_arguments:;
                            argv[i], 0);
      if(ret <= 0)
        goto ex;
+   } else if(strcmp(argv[i], "-name_limit_blocker") == 0) {
+     if(i + 1 >= end_idx)
+       goto not_enough_arguments;
+     i++;
+     sscanf(argv[i], "%d", &count);
+     if(count < 64 || count > 255) {
+       sprintf(xorriso->info_text,
+               "-findi: wrong length with -name_limit_blocker [64...255]");
+       goto sorry_ex;
+     }
+     ret= Findjob_set_num_filter(job, 24, count, 0, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-exec")==0) {
      if(i+1>=end_idx) {
 not_enough_exec_arguments:;
@@ -1415,7 +1494,7 @@ int Xorriso_option_fs(struct XorrisO *xorriso, char *size, int flag)
 }
 
 
-/* Optionis -getfacl alias -getfacli, -getfacl_r alias -getfacl_ri
+/* Commands -getfacl alias -getfacli, -getfacl_r alias -getfacl_ri
             -getfattr alias getfattri
 */
 /* @param flag bit0= recursive -getfacl_r
@@ -1631,20 +1710,24 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 those lists empty. Defaulty entry in \"risky\" is \"/dev\".",
 "  -grow_blindly \"off\"|predicted_nwa",
 "                 Switch between modifying and blind growing.",
+"  -scsi_dev_family \"default\"|\"sr\"|\"scd\"|\"sg\"",
+"                 Linux specific: Choose device file type.",
 "  -read_speed number[\"k/s\"|\"m/s\"|\"[x]CD\"|\"[x]DVD\"|\"[x]BD\"]",
 "                 Set the read speed. Default is \"none\" = do not set speed",
 "                 before reading.",
 "  -load \"session\"|\"track\"|\"lba\"|\"sbsector\"|\"volid\"|\"auto\" id",
-"                 Load a particular (outdated) ISO image from a -dev or",
+"                 Load a particular (outdated) ISO session from a -dev or",
 "                 -indev which hosts more than one session.",
 "  -displacement [-]block_address",
 "                 When loading ISO tree or reading data files compensate a",
 "                 displacement versus the start address for which the image",
 "                 was prepared.",
+"  -read_fs \"any\"|\"norock\"|\"nojoliet\"|\"ecma119\"",
+"                 Specify which kind of filesystem tree to load if present.",
 "  -rom_toc_scan \"on\"|\"force\"|\"off\"[:\"emul_on\"|\"emul_off\"]",
 "                [:\"emul_wide\"|\"emul_narrow\"]",
 "                 Enable scanning for ISO sessions on read-only drives/media",
-"                 resp. on overwriteable media with emulated TOC.",
+"                 and on overwriteable media with emulated TOC.",
 "  -calm_drive \"in\"|\"out\"|\"all\"|\"on\"|\"off\"",
 "                 Reduce drive noise until it gets actually used again.",
 "  -assert_volid pattern severity",
@@ -1660,21 +1743,24 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -local_charset name",
 "                 Override system assumption of the local character set name.",
 "  -hardlinks mode[:mode ...]",
-"                 Enable resp. disable recording and restoring of hard links.",
+"                 Enable or disable recording and restoring of hard links.",
 "                 Modes are \"on\", \"off\", \"perform_update\",",
 "                 \"without_update\", \"discard_extract\",",
 "                 \"cheap_sorted_extract\", \"normal_extract\"",
 "  -acl \"on\"|\"off\"",
-"                 Enable resp. disable reading and writing of ACLs.",
+"                 Enable or disable reading and writing of ACLs.",
 "  -xattr \"on\"|\"off\"",
-"                 Enable resp. disable reading and writing of xattr.",
+"                 Enable or disable reading and writing of xattr.",
+"  -md5 \"on\"|\"all\"|\"off\"",
+"                 Enable or disable processing of MD5 checksums.",
 "  -for_backup",
 "                 Shortcut for: -hardlinks on -acl on -xattr on -md5 on",
+"  -ecma119_map \"unmapped\"|\"stripped\"|\"uppercase\"|\"lowercase\"",
+"                 Choose conversion of file names if neither Rock Ridge",
+"                 nor Joliet is present in the loaded ISO session.",
 "  -disk_dev_ino \"on\"|\"ino_only\"|\"off\"",
-"                 Enable resp. disable recording of disk file dev_t and ino_t",
+"                 Enable or disable recording of disk file dev_t and ino_t",
 "                 and their use in file comparison.",
-"  -md5 \"on\"|\"all\"|\"off\"",
-"                 Enable resp. disable processing of MD5 checksums.",
 "  -scdbackup_tag list_path record_name",
 "                 Enable production of scdbackup tag with -md5 on",
 "  -ban_stdio_write",
@@ -1684,7 +1770,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -data_cache_size number_of_tiles blocks_per_tile",
 "                 Adjust size and granularity of the data read cache.",
 "  -blank [\"force:\"]\"fast\"|\"all\"|\"deformat\"|\"deformat_quickest\"",
-"                 Blank medium resp. invalidate ISO image on medium.",
+"                 Blank medium or invalidate ISO image on medium.",
 "                 Prefix \"force:\" overrides medium evaluation.",
 "  -close_damaged \"as_needed\"|\"force\"",
 "                 Close track and session of damaged medium.",
@@ -1733,14 +1819,16 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 to which deep subtrees will get relocated if -compliance",
 "                 is set to \"deep_paths_off\".",
 "  -boot_image \"any\"|\"isolinux\"|\"grub\"",
-"              |\"discard\"|\"keep\"|\"patch\"|\"dir=\"|\"bin_path=\"",
+"              |\"discard\"|\"keep\"|\"patch\"|\"replay\"",
+"              |\"dir=\"|\"bin_path=\"",
 "              |\"cat_path=\"|\"cat_hidden=on|iso_rr|joliet|off\"",
 "              |\"load_size=\"|\"boot_info_table=\"",
 "              |\"grub2_boot_info=\"|\"grub2_mbr=\"|\"partition_offset=\"",
 "              |\"partition_hd_cyl=\"|\"partition_sec_hd=\"",
-"              |\"partition_cyl_align=\"",
+"              |\"partition_cyl_align=\"|\"mbr_force_bootable=\"",
 "              |\"system_area=\"|\"partition_table=on|off\"",
 "              |\"partition_entry=\"|\"appended_part_as=\"",
+"              |\"part_like_isohybrid=\"",
 "              |\"chrp_boot_part=on|off=\"|\"prep_boot_part=\"",
 "              |\"efi_boot_part=\"|\"efi_boot_part=--efi-boot-image\"",
 "              |\"mips_path=\"|\"mipsel_path=\"|\"mips_discard\"",
@@ -1750,7 +1838,9 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              |\"hppa_discard\"|\"alpha_boot=\"|\"alpha_discard\"",
 "              |\"hfsplus_serial=\"|\"hfsplus_block_size=\"",
 "              |\"apm_block_size=\"|\"show_status\"",
-"                 Whether to discard or keep an exiting El Torito boot image.",
+"                 Whether to discard or keep an exiting El Torito boot image,",
+"                 or to freshly set up boot equipment. \"replay\" performs",
+"                 the commands proposed by -report_system_area \"cmd\".",
 "                 ISOLINUX can be made bootable by dir=/ or dir=/isolinux",
 "                 or dir=/boot/isolinux. Others, like GRUB, by bin_path=...",
 "                 and cat_path=...",
@@ -1794,22 +1884,27 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -list_speeds   Show media specific list of write speed descriptors.",
 "",
 "  -list_profiles \"in\"|\"out\"|\"all\"",
-"                 Show list of media types supported by indev resp. outdev.",
+"                 Show list of media types supported by indev and/or outdev.",
 "  -print_size    Print the foreseeable consumption by next -commit.",
 "",
 "  -tell_media_space",
 "                 Print foreseeable available space on output medium",
 "  -pvd_info      Print various id strings of the loaded ISO image.",
 "",
-"  -report_el_torito \"plain\"|\"help\"",
+"  -report_el_torito \"plain\"|\"help\"|\"cmd\"|\"as_mkisofs\"",
 "                 \"plain\" prints information about the El Torito boot catalog",
 "                 and boot images of the loaded ISO image.",
 "                 \"help\" prints an explanation of the output format.",
+"                 \"cmd\" and \"as_mkisofs\" propose commands to reproduce",
+"                 the boot equipment reported by -report_el_torito \"plain\"",
+"                 and -report_system_area \"plain\"",
 "  -report_system_area \"plain\"|\"help\"|\"gpt_crc_of:\"disk_path",
+"                      |\"cmd\"|\"as_mkisofs\"",
 "                 \"plain\" prints information about recognized data",
-"                 in the System Area of the loaded ISO image: MBR, GPT, ...",
 "                 \"help\" prints an explanation of the output format.",
+"                 in the System Area of the loaded ISO image: MBR, GPT, ...",
 "                 \"gpt_crc_of:\" prints GPT CRC of file disk_path.",
+"                 For \"cmd\" and \"as_mkisofs\" see -report_el_torito.",
 "",
 "Commands with variable length path list [...] need the list delimiter text",
 "as end mark if they are followed by another command. By default this",
@@ -1827,16 +1922,20 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "iso_rr_path is the Rock Ridge name of a file object in the ISO image.",
 "pathspec is either a disk_path or (if allowed) a pair: iso_rr_path=disk_path",
 "Commands marked by [***] have variable length parameter lists and perform",
-"pattern expansion if enabled by -iso_rr_pattern resp. -disk_pattern.",
+"pattern expansion if enabled by -iso_rr_pattern or -disk_pattern.",
 "",
-"  -pathspecs \"on\"|\"off\"  Allow or disallow pathspecs of form ",
-"                 iso_rr_path=disk_path . Only \"off\" allows eventual",
-"                 -disk_pattern expansion.",
+"  -pathspecs \"on\"|\"off\"|\"as_mkisofs\"",
+"                 Allow or disallow pathspecs of form iso_rr_path=disk_path",
+"                 Only \"off\" allows -disk_pattern expansion.",
+"  -file_name_limit number",
+"                 Set truncation size for file names [64 ... 255].",
+"  -file_size_limit value [...]",
+"                 Set limit for file content size. One or more numbers to add.",
 "  -add pathspec [...] | disk_path [***]",
 "                 Insert the given files or directory trees from",
 "                 filesystem into the ISO image. Much like mkisofs.",
 "  -add_plainly \"none\"|\"unknown\"|\"dashed\"|\"any\"",
-"                 Whether to add lonely arguments as pathspec resp. disk_path.",
+"                 Whether to add lonely arguments as pathspec or disk_path.",
 "  -path_list disk_path",
 "                 Like -add but read the pathspecs from file disk_path.",
 "  -quoted_path_list disk_path",
@@ -1919,6 +2018,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                   -lba_range start count, -damaged, -has_acl, -has_xattr,",
 "                   -has_aaip, -has_filter, -has_md5, -has_any_xattr,",
 "                   -has_hfs_crtp, -has_hfs_bless, -bad_outname,",
+"                   -name_limit_blocker",
 "                   -prune, -decision yes|no, -true, -false",
 "                 Operators: -not, -or, -and, -sub, (, -subend, ),",
 "                   -if, -then, -elseif, -else, -endif",
@@ -2028,6 +2128,11 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Set the burn speed. Default is \"max\" = maximum speed.",
 "  -stream_recording \"on\"|\"off\"",
 "                 Try to circumvent slow checkread on DVD-RAM, BD-RE, BD-R.",
+"  -modesty_on_drive [\"on\"|\"off\"|min_percent_number]",
+"                    [:\"min_percent=\"number][:\"max_percent=\"number]",
+"                    [:\"min_usec=\"number][:\"max_usec\"=number]",
+"                    [:\"timeout_sec=\"number]",
+"                 Keep drive buffer hungry to ease concurrent burn run.",
 "  -dvd_obs \"default\"|\"32k\"|\"64k\"",
 "                 Set number of bytes per DVD/BD write operation.",
 "  -stdio_sync \"on\"|\"off\"|\"end\"|number",
@@ -2036,8 +2141,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -fs number[\"k\"|\"m\"]",
 "                 Set the size of the fifo buffer. (Default is 4m)",
 "  -eject \"in\"|\"out\"|\"all\"",
-"                 Immediately eject the medium in -indev, resp. -outdev,",
-"                 resp. both.",
+"                 Immediately eject the medium in -indev, -outdev, or both.",
 "",
 "Navigation commands:",
 "",
@@ -2084,9 +2188,9 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 local filesystem which match one of the shell parser",
 "                 patterns.",
 "  -dus pattern [***]  like -du but summing up subdirectories without",
-"                 listing them explicitely.",
+"                 listing them explicitly.",
 "  -dusx pattern [***]  like -dux but summing up subdirectories without",
-"                 listing them explicitely.",
+"                 listing them explicitly.",
 "",
 "  -findx disk_path [-name pattern] [-type t] [-exec action [params]]",
 "                 Like -find but operating on local filesystem. Most -exec",
@@ -2182,12 +2286,12 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "",
 "General commands:",
 "  -help       Print this text",
-"  -abort_on severity   Set the threshhold for events to abort the program.",
+"  -abort_on severity   Set the threshold for events to abort the program.",
 "              Useful severities: NEVER, ABORT, FATAL, FAILURE, SORRY, WARNING",
-"  -return_with severity exit_value   Set the threshhold for events to return",
+"  -return_with severity exit_value   Set the threshold for events to return",
 "              at program end the given exit_value even if not aborted.",
 "              exit_value may be 0 or 32 to 63.",
-"  -report_about severity   Set the threshhold for events to be reported.",
+"  -report_about severity   Set the threshold for events to be reported.",
 "              Use -abort_on severities or: HINT, NOTE, UPDATE, DEBUG, ALL",
 "  -signal_handling \"on\"|\"off\"|\"sig_dfl\"|\"sig_ign\"",
 "              Handling of signals. Default \"on\" uses libburn handler.",
@@ -2273,7 +2377,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -print_mark text",
 "              Print a text to mark channel.",
 "  -prompt text",
-"              Wait for Enter key resp. for a line of input at stdin.",
+"              Wait for Enter key or for a line of input at stdin.",
 "  -sleep number",
 "              Do nothing during the given number of seconds.",
 "  -errfile_log mode path|channel",
