@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -215,9 +215,10 @@ int Xorriso_option_lnsi(struct XorrisO *xorriso, char *target, char *path,
                         int flag)
 {
  int ret;
- char *eff_path= NULL;
+ char *eff_path= NULL, *buffer= NULL, *namept;
 
  Xorriso_alloc_meM(eff_path, char, SfileadrL);
+ Xorriso_alloc_meM(buffer, char, SfileadrL);
 
  ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, path, eff_path, 1);
  if(ret < 0)
@@ -231,12 +232,16 @@ int Xorriso_option_lnsi(struct XorrisO *xorriso, char *target, char *path,
  ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, path, eff_path, 2);
  if(ret < 0)
    {ret= 0; goto ex;}
- ret= Xorriso_graft_in(xorriso, NULL, target, eff_path, (off_t) 0, (off_t) 0,
+ ret= Xorriso_truncate_path_comps(xorriso, target, buffer, &namept, 0);
+ if(ret < 0)
+   {ret= 0; goto ex;}
+ ret= Xorriso_graft_in(xorriso, NULL, namept, eff_path, (off_t) 0, (off_t) 0,
                        1024);
  if(ret <= 0)
    {ret= 0; goto ex;}
  ret= 1;
 ex:;
+ Xorriso_free_meM(buffer);
  Xorriso_free_meM(eff_path);
  return(ret);
 }
@@ -315,19 +320,27 @@ logfile_wrong_form:;
 /* Options -ls  alias -lsi   and -lsl  alias -lsli
        and -lsd alias -lsdi  and -lsdl alias -lsdli
        and -du  alias -dui   and -dus  alias -dusi
-   @param flag bit0= long format (-lsl , -du)
+   @param flag bit0= long format (-lsl , -du, not -dus, not -ls)
                bit1= do not expand patterns but use literally
-               bit2= du rather than ls
-               bit3= list directories as themselves (ls -d) 
+               bit2= -du rather than -ls
+               bit3= list directories as themselves (-lsd) 
 */
 int Xorriso_option_lsi(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag)
 {
- int ret, end_idx, filec= 0, nump, i;
+ int ret, end_idx, filec= 0, nump, i, star= 1;
  char **filev= NULL, **patterns= NULL;
  off_t mem= 0;
  struct stat stbuf;
 
+ if(flag & 4) {
+   if(!(flag & 1))
+     star= 0;
+ } else {
+   if(flag & 8)
+     star= 0;
+ }
+
  end_idx= Xorriso_end_idx(xorriso, argc, argv, *idx, 1);
  if(xorriso->do_iso_rr_pattern==0)
    flag|= 2;
@@ -357,19 +370,22 @@ no_memory:;
      {ret= -1; goto ex;}
    }
    nump= 1;
-   if(flag&8)
-     patterns[0]= ".";
-   else 
+   if(star)
      patterns[0]= "*";
+   else 
+     patterns[0]= ".";
    flag&= ~2;
  } else {
    patterns= calloc(nump, sizeof(char *));
    if(patterns==NULL)
      goto no_memory;
    for(i= 0; i<nump; i++) {
-     if(argv[i + *idx][0]==0)
-       patterns[i]= "*";
-     else
+     if(argv[i + *idx][0]==0) {
+       if(star)
+         patterns[i]= "*";
+       else
+         patterns[i]= ".";
+     } else
        patterns[i]= argv[i + *idx];
    }
  }
@@ -377,7 +393,7 @@ no_memory:;
  if((flag & 1) && !(xorriso->ino_behavior & 32)) {
    ret= Xorriso_make_hln_array(xorriso, 0); /* for stbuf.st_nlink */
    if(ret < 0)
-     return(ret);
+     goto ex;
  }
  if(flag&2) {
    ret= Xorriso_ls_filev(xorriso, xorriso->wdi, nump, argv + (*idx), mem,
@@ -486,7 +502,7 @@ ex:;
 /* Option -map , -map_single */
 /* @param flag bit0=do not report the added item
                bit1=do not reset pacifier, no final pacifier message
-               bit5=eventually do not insert directory tree
+               bit5= -map_single: do not insert directory tree
 */
 int Xorriso_option_map(struct XorrisO *xorriso, char *disk_path,
                        char *iso_path, int flag)
@@ -788,6 +804,97 @@ ex:;
 }
 
 
+int Xorriso_option_modesty_on_drive(struct XorrisO *xorriso, char *mode,
+                                    int flag)
+{
+ char *npt, *cpt, *ppt;
+ int l, num, set_min;
+
+ npt= cpt= mode;
+ for(; npt!=NULL; cpt= npt+1) {
+   npt= strchr(cpt,':');
+   if(npt==NULL)
+     l= strlen(cpt);
+   else
+     l= npt-cpt;
+   if(l == 0)
+ continue;
+   if(l == 3 && strncmp(cpt, "off", l) == 0) {
+     xorriso->modesty_on_drive= 0;
+   } else if(l == 1 && strncmp(cpt, "0", l) == 0) {
+     xorriso->modesty_on_drive= 0;
+   } else if(l == 2 && strncmp(cpt, "on", l) == 0) {
+     xorriso->modesty_on_drive= 1;
+   } else if(l == 1 && strncmp(cpt, "1", l) == 0) {
+     xorriso->modesty_on_drive= 1;
+   } else if(l == 2 && strncmp(cpt, "-1", l) == 0) {
+     ;
+   } else if(*cpt >= '1' && *cpt <= '9') {
+     ppt= cpt;
+     set_min= 2;
+set_size_percent:;
+     sscanf(ppt, "%d", &num);
+     if(num == -1) {
+       ;
+     } else if(num < 25) {
+bad_percent:;
+       sprintf(xorriso->info_text, "-modesty_on_drive: percentage out of range [25 to 100]");
+       Text_shellsafe(cpt, xorriso->info_text, 1);
+       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+       return(0);
+     } else if(num > 100) {
+       goto bad_percent;
+     }
+     if(set_min == 2) {
+       xorriso->modesty_on_drive= 1;
+     }
+     if(set_min)
+       xorriso->min_buffer_percent= num;
+     else
+       xorriso->max_buffer_percent= num;
+   } else if(l >= 12 && strncmp(cpt, "min_percent=", 12) == 0) {
+     ppt= cpt + 12;
+     set_min= 1;
+     goto set_size_percent;
+   } else if(l >= 12 && strncmp(cpt, "max_percent=", 12) == 0) {
+     ppt= cpt + 12;
+     set_min= 0;
+     goto set_size_percent;
+   } else if(l >= 9 && strncmp(cpt, "min_usec=", 9) == 0) {
+     ppt= cpt + 9;
+     set_min= 1;
+set_sec:;
+     num= -1;
+     sscanf(ppt, "%d", &num);
+     if(num < 0)
+       num= 0;
+     if(set_min == 2)
+       xorriso->max_buffer_usec= num;
+     else if(set_min == 1)
+       xorriso->min_buffer_usec= num;
+     else if(set_min == 0)
+       xorriso->max_buffer_percent= num;
+     else
+       xorriso->buffer_timeout_sec= num;
+   } else if(l >= 9 && strncmp(cpt, "max_usec=", 9) == 0) {
+     ppt= cpt + 9;
+     set_min= 2;
+     goto set_sec;
+   } else if(l >= 12 && strncmp(cpt, "timeout_sec=", 12) == 0) {
+     ppt= cpt + 12;
+     set_min= -1;
+     goto set_sec;
+   } else {
+     sprintf(xorriso->info_text, "-modesty_on_drive: unknown mode ");
+     Text_shellsafe(cpt, xorriso->info_text, 1);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+ }
+ return(1);
+}
+
+
 /* Options -mount , -mount_cmd , -session_string */
 /* @param bit0= -mount_cmd: print mount command to result channel rather
                             than performing it
@@ -878,10 +985,10 @@ int Xorriso_option_move(struct XorrisO *xorriso, char *origin, char *dest,
 
  ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, origin, eff_origin, 0);
  if(ret <= 0)
-   goto ex;
- ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, dest, eff_dest, 0);
- if(ret <= 0)
-   goto ex;
+   {ret= 0; goto ex;}
+ ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, dest, eff_dest, 2);
+ if(ret < 0)
+   {ret= 0; goto ex;}
  ret= Xorriso_rename(xorriso, NULL, eff_origin, eff_dest, 0);
  if(ret <= 0)
    goto ex;
@@ -904,7 +1011,9 @@ int Xorriso_option_msg_op(struct XorrisO *xorriso, char *what, char *arg,
  msd_mem= xorriso->msg_sieve_disabled;
 
  ret= 1;
- if(strcmp(what, "parse") == 0 || strcmp(what, "parse_bulk") == 0) {
+ if(strcmp(what, "parse") == 0 || strcmp(what, "parse_silently") == 0 ||
+    strcmp(what, "parse_bulk") == 0 ||
+    strcmp(what, "parse_bulk_silently") == 0) {
    ret= Xorriso_parse_line(xorriso, arg, "", "", 5, &argc, &argv, 0);
    prefix= "";
    if(argc > 0) 
@@ -921,12 +1030,14 @@ int Xorriso_option_msg_op(struct XorrisO *xorriso, char *what, char *arg,
    input_lines= 1;
    if(argc > 4)
      sscanf(argv[4], "%d", &input_lines);
-   if(strcmp(what, "parse") == 0) {
+   if(strcmp(what, "parse") == 0 || strcmp(what, "parse_silently") == 0) {
      ret= Xorriso_msg_op_parse(xorriso, "", prefix, separators,
-                               max_words, pflag, input_lines, 0);
+                               max_words, pflag, input_lines,
+                               (strcmp(what, "parse_silently") == 0));
    } else {
      ret= Xorriso_msg_op_parse_bulk(xorriso, prefix, separators,
-                               max_words, pflag, input_lines, 0);
+                               max_words, pflag, input_lines,
+                               (strcmp(what, "parse_bulk_silently") == 0));
    }
    if(ret <= 0)
      goto ex;
@@ -1086,6 +1197,48 @@ ex:;
 }
 
 
+/* Option -named_pipe_loop */
+int Xorriso_option_named_pipe_loop(struct XorrisO *xorriso, char *mode,
+                                   char *stdin_pipe, char *stdout_pipe,
+                                   char *stderr_pipe, int flag)
+{
+ char *pipe_paths[3], *cpt, *npt;
+ int ret, hflag= 0, l;
+
+ npt= mode;
+ for(cpt= mode; npt != NULL; cpt= npt + 1) {
+   npt= strchr(cpt, ':');
+   if(npt==NULL)
+     l= strlen(cpt);
+   else
+     l= npt-cpt;
+   if(l==0) {
+     ;
+   } else if(strncmp(cpt, "-", l) == 0) {
+     ;
+   } else if(strncmp(cpt, "cleanup", l) == 0) {
+     hflag|= 1;
+   } else if(strncmp(cpt, "keep", l) == 0) {
+     hflag&= ~1;
+   } else if(strncmp(cpt, "buffered", l) == 0) {
+     hflag|= 2;
+   } else if(strncmp(cpt, "direct", l) == 0) {
+     hflag&= ~2;
+   } else { 
+     sprintf(xorriso->info_text, "-named_pipe_loop: unknown mode in '%s'",
+             mode);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+ }
+ pipe_paths[0]= stdin_pipe;
+ pipe_paths[1]= stdout_pipe;
+ pipe_paths[2]= stderr_pipe;
+ ret= Xorriso_named_pipe_loop(xorriso, pipe_paths, hflag);
+ return(ret);
+}
+
+
 /* Option -no_rc */
 int Xorriso_option_no_rc(struct XorrisO *xorriso, int flag)
 {
@@ -1094,7 +1247,7 @@ int Xorriso_option_no_rc(struct XorrisO *xorriso, int flag)
 }
 
 
-/* Option -not_leaf , (-hide_disk_leaf resp. -as mkisofs -hide) */
+/* Option -not_leaf , (-hide_disk_leaf , -as mkisofs -hide) */
 /* @param flag  bit0-bit5= hide rather than adding to disk_exclusions
                   bit0= add to iso_rr_hidings
                   bit1= add to joliet_hidings
@@ -1187,9 +1340,9 @@ int Xorriso_option_not_list(struct XorrisO *xorriso, char *adr, int flag)
  }
  ret= 1;
 ex:;
+ Xorriso_read_lines(xorriso, fp, &linecount, &argc, &argv, 2);
  if(fp != NULL && fp != stdin)
    fclose(fp);
- Xorriso_read_lines(xorriso, fp, &linecount, &argc, &argv, 2);
  if(ret<=0) {
    sprintf(xorriso->info_text, "Aborted reading of file ");
    Text_shellsafe(adr, xorriso->info_text, 1);
@@ -1266,7 +1419,7 @@ ex:;
 }
 
 
-/* Option -not_paths , (-hide_disk_paths resp. -as mkisofs -hide) */
+/* Option -not_paths , (-hide_disk_paths , -as mkisofs -hide) */
 /* @param flag  bit0= add to iso_rr_hidings rather than disk_exclusions
                 bit1= add to joliet_hidings rather than disk_exclusions
                 bit2= enable disk pattern expansion regardless of -disk_pattern
