@@ -2,7 +2,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -668,10 +668,10 @@ int Xorriso_tree_restore_node(struct XorrisO *xorriso, IsoNode *node,
    break;
      }
    }  
-   if(write_fd > 0)
+   if(write_fd != -1)
      close(write_fd);
    write_fd= -1;
-   if(todo > 0 && xorriso->extract_error_mode == 2 && open_path_pt != NULL) {
+   if(todo > 0 && xorriso->extract_error_mode == 2) {
      unlink(open_path_pt);
      target_deleted= 1;
    }
@@ -779,7 +779,7 @@ restore_properties:;
  if(todo < 0)
    ret= 0;
 ex:;
- if(write_fd >= 0) {
+ if(write_fd != -1) {
    close(write_fd);
    if(ret <= 0 && xorriso->extract_error_mode == 2 && open_path_pt != NULL)
      unlink(open_path_pt);
@@ -991,7 +991,7 @@ int Xorriso_restore_disk_object(struct XorrisO *xorriso,
  int ret, i, split_count= 0, partno, total_parts, leaf_is_split= 0;
  int record_hl_path= 0, node_idx, cannot_register= 0;
  off_t total_bytes;
- char *part_name, *part_path= NULL, *img_path_pt;
+ char *part_name, *part_path= NULL, *img_path_pt= NULL;
  IsoImage *volume;
  IsoNode *part_node, *first_part_node= NULL;
  struct SplitparT *split_parts= NULL;
@@ -1211,7 +1211,7 @@ int Xorriso_restore_tree(struct XorrisO *xorriso, IsoDir *dir,
  IsoNode *node;
  IsoDirIter *iter= NULL;
  IsoNode **node_array= NULL;
- int node_count, node_idx;
+ int node_count= 0, node_idx;
  int ret, source_is_dir, fret, was_failure= 0;
  int do_not_dive, source_is_split= 0, len_dp, len_ip, stbuf_ret, hflag, hret;
  char *name, *disk_name, *leaf_name, *srcpt, *stbuf_src= "";
@@ -1987,7 +1987,7 @@ int Xorriso_read_file_data(struct XorrisO *xorriso, IsoNode *node,
 
  if(Sfile_str(job->data_to_path, disk_path, 0) <= 0)
    {ret= -1; goto ex;}
- Xorriso_open_job_data_to(xorriso, job, 0);
+ ret= Xorriso_open_job_data_to(xorriso, job, 0);
  if(ret <= 0)
    goto ex;
 
