@@ -2,7 +2,7 @@
 /* xorriso - libisoburn higher level API which creates, loads, manipulates
              and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -48,6 +48,11 @@
 #ifndef Xorriso_includeD
 #define Xorriso_includeD yes
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 /** Opaque handle of the xorriso runtime context */
 struct XorrisO; 
 
@@ -69,7 +74,7 @@ struct XorrisO;
 */
 #define Xorriso_header_version_majoR  1
 #define Xorriso_header_version_minoR  4
-#define Xorriso_header_version_micrO  2
+#define Xorriso_header_version_micrO  4
 
 
 /** If needed: Something like ".pl01" to indicate a bug fix. Normally empty.
@@ -568,7 +573,7 @@ char *Xorriso__severity_list(int flag);
     @param xorriso    The environment handle
     @param severity   The severity text matching the current problem status
     @param flag       Unused yet. Submit 0.
-    @return           The severity rank number. 0= no problem occured.
+    @return           The severity rank number. 0= no problem occurred.
 */
 int Xorriso_get_problem_status(struct XorrisO *xorriso, char severity[80],
                                int flag);
@@ -1993,6 +1998,12 @@ int Xorriso_option_rr_reloc_dir(struct XorrisO *xorriso, char *name, int flag);
 /* @since 0.4.4 */
 int Xorriso_option_scdbackup_tag(struct XorrisO *xorriso, char *list_path,
                                  char *record_name, int flag);
+
+/* Command -scsi_dev_family */
+/* @since 1.4.4 */
+int Xorriso_option_scsi_dev_family(struct XorrisO *xorriso, char *mode,
+                                   int flag);
+
 /* Command -scsi_log */
 /* @since 0.5.0 */
 int Xorriso_option_scsi_log(struct XorrisO *xorriso, char *mode, int flag);
@@ -2158,6 +2169,9 @@ int Xorriso_option_xattr(struct XorrisO *xorriso, char *mode, int flag);
 /* @since 0.3.8 */
 int Xorriso_option_zisofs(struct XorrisO *xorriso, char *mode, int flag);
 
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
 
 #endif /* Xorriso_includeD */
 
