@@ -2,7 +2,7 @@
 /* Command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -282,6 +282,8 @@ struct XorrisO { /* the global context of xorriso */
 
  int drives_exclusive;   /* burn_preset_device_open() param exclusive */
 
+ int linux_scsi_dev_family; /* 0= default, 1= sr, 2= scd, (3= st), 4= sg */
+
  int early_stdio_test;   /* For burn_allow_drive_role_4():
                             bit1= Test whether a stdio drive can be opened for
                                   read-write resp. read-only resp. write only.
@@ -411,6 +413,7 @@ struct XorrisO { /* the global context of xorriso */
                           1= appy --efi-boot parameters when attaching to img */
 
  char system_area_disk_path[SfileadrL];
+ int system_area_clear_loaded;
  int system_area_options;             /* bit0= "GRUB protective msdos label"
                                                (a simple partition table)
                                          bit1= isohybrid boot image pointer
@@ -450,6 +453,11 @@ struct XorrisO { /* the global context of xorriso */
                                                Little-endian 8-byte.
                                                Should be combined with
                                                options bit0.
+                                         bit15= Only with MBR:
+                                               Enforce MBR "bootable/active"
+                                               flag. In worst case by dummy
+                                               partition of type 0x00 which
+                                               occupies block 0.
                                        */
  int patch_system_area;               /* Bits as of system_area_options.
                                          to be applied to the loaded system
@@ -477,7 +485,11 @@ struct XorrisO { /* the global context of xorriso */
  uint8_t appended_part_types[Xorriso_max_appended_partitionS];
  /* If 1: With appended partitions: create protective MBR and mark by GPT */
  int appended_as_gpt;
+ /* If 1: With appended partitions: mark by APM */
+ int appended_as_apm;
 
+ /* If 1: Apply isohybrid gestures to non-isohybrid situations */
+ int part_like_isohybrid;
 
  /* Eventual name of the non-ISO aspect of the image. E.g. SUN ASCII label.
  */
@@ -528,7 +540,11 @@ struct XorrisO { /* the global context of xorriso */
  char loaded_boot_cat_path[SfileadrL];
 
  /* XORRISO options */
- int allow_graft_points;
+ int allow_graft_points;   /* if not zero: yes
+                              special options:
+                              bit1= unescape '\\' too,
+                                    continue unescaping after first real '='
+                            */
 
  int allow_restore; /* -2=disallowed until special mode "unblock"
                        -1=permanently disallowed
