@@ -4,7 +4,7 @@
    a command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  1
 #define xorriso_libisoburn_req_minor  4
-#define xorriso_libisoburn_req_micro  0
+#define xorriso_libisoburn_req_micro  4
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -217,7 +217,7 @@ int Xorriso_set_abort_severity(struct XorrisO *xorriso, int flag);
 int Xorriso_report_lib_versions(struct XorrisO *xorriso, int flag);
 
 /* @return 0= stbuf content is valid ,
-          -1 = path not found , -2 = severe error occured
+          -1 = path not found , -2 = severe error occurred
 */
 int Xorriso_iso_lstat(struct XorrisO *xorriso, char *path, struct stat *stbuf,
                       int flag);
@@ -622,6 +622,8 @@ int Xorriso_hfsplus_bless(struct XorrisO *xorriso, char *path,
 
 int Xorriso_pretend_full_disc(struct XorrisO *xorriso, int flag);
 
+int Xorriso_scsi_dev_family(struct XorrisO *xorriso, int flag);
+
 
 int Xorriso_apply_sort_file(struct XorrisO *xorriso, char *path, int flag);
 
@@ -630,6 +632,13 @@ int Xorriso_report_system_area(struct XorrisO *xorriso, char *form, int flag);
 int Xorriso_external_filter_banned(struct XorrisO *xorriso, char *purpose,
                                    int flag);
 
+int Xorriso_set_file_name_limit(struct XorrisO *xorriso, int value, int flag);
+
+int Xorriso_truncate_path_comps(struct XorrisO *xorriso, char *path,
+                                char *buffer, char **resultpt, int flag);
+
+int Xorriso_graftable_pathspec(struct XorrisO *xorriso, char *in_pathspec,
+                               char *pathspec, int flag);
 
 #endif /* Xorrisoburn_includeD */
 
